@@ -230,18 +230,24 @@
 #define WinStateFixedPosition  (1 << 10)  /* fixed position on virtual desktop*/
 #define WinStateArrangeIgnore  (1 << 11)  /* ignore for auto arranging */
 //#define WinStateDocked         (1 << 9) /* docked, ignore my area for maximizing */
+#define WinStateWasHidden      (1 << 29)  /* was hidden when parent was minimized/hidden */
+#define WinStateWasMinimized   (1 << 30)  /* was minimized when parent was minimized/hidden */
 #define WinStateWithdrawn      (1 << 31)  /* managed, but not available to user */
 
 #define WIN_STATE_ALL (WinStateAllWorkspaces | WinStateMinimized | WinStateMaximizedVert | WinStateMaximizedHoriz | WinStateHidden | WinStateRollup)
 
 
 /* hints */
-#define XA_WIN_HINTS           "_WIN_HINTS"
-#define WinHintsSkipFocus      (1 << 0)
-#define WinHintsSkipWindowMenu (1 << 1)
-#define WinHintsSkipTaskBar    (1 << 2)
-#define WinHintsGroupTransient (1 << 3)
-#define WinHintsDockHorizontal (1 << 6)   /* docked horizontally */
+#define XA_WIN_HINTS		"_WIN_HINTS"
+#define WinHintsSkipFocus	(1 << 0)
+#define WinHintsSkipWindowMenu	(1 << 1)
+#define WinHintsSkipTaskBar	(1 << 2)
+#define WinHintsGroupTransient	(1 << 3)
+#define WinHintsFocusOnClick	(1 << 4) /* app only accepts focus when clicked */
+#define WinHintsDoNotCover	(1 << 5) /* attempt to not cover this window */
+#define WinHintsDockHorizontal	(1 << 6) /* docked horizontally */
+
+
 
 /* Type CARD32[2]
  *      additional window hints
