@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 #include "yfull.h"
@@ -54,8 +54,10 @@ void YFrameWindow::updateMenu() {
         windowMenu->disableCommand(actionSize);
     if (!(func & ffMinimize))
         windowMenu->disableCommand(actionMinimize);
+#ifndef	CONFIG_PDA
     if (!(func & ffHide))
         windowMenu->disableCommand(actionHide);
+#endif
     if (!(func & ffRollup) || !titlebar()->visible())
         windowMenu->disableCommand(actionRollup);
     if (!(func & ffMaximize))
@@ -150,16 +152,16 @@ void YFrameWindow::setShape() {
                                         ShapeBounding,
                                         0, 0, rect, nrect,
                                         ShapeSet, Unsorted);
-            XShapeCombineShape (app->display(), handle(),
-                                ShapeBounding,
-                                borderX(),
-                                borderY()
+            XShapeCombineShape(app->display(), handle(),
+                               ShapeBounding,
+                               borderX(),
+                               borderY()
 #ifndef TITLEBAR_BOTTOM
-                                + titleY()
+                               + titleY()
 #endif
-                                ,
-                                client()->handle(),
-                                ShapeBounding, nrect ? ShapeUnion : ShapeSet);
+                               ,
+                               client()->handle(),
+                               ShapeBounding, nrect ? ShapeUnion : ShapeSet);
         }
     }
 }
@@ -200,8 +202,8 @@ void YFrameWindow::configure(int x, int y, unsigned int width, unsigned int heig
         setShape();
 #endif
 
-    if (getLayer() == WinLayerDock)
-        manager->updateWorkArea();
+    if (doNotCover())
+	manager->updateWorkArea();
 }
 
 void YFrameWindow::layoutTitleBar() {
@@ -309,8 +311,17 @@ void YFrameWindow::layoutButtons() {
         else
             fDepthButton->hide();
 
+#ifdef CONFIG_LOOK_PIXMAP
+	const int pi(focused() ? 1 : 0);
+#endif
+
     if (titleButtonsLeft) {
-        int xPos = 0;
+#ifdef CONFIG_LOOK_PIXMAP
+        int xPos(titleJ[pi] ? titleJ[pi]->width() : 0);
+#else	
+        int xPos(0);
+#endif
+
         for (const char *bc = titleButtonsLeft; *bc; bc++) {
             YFrameButton *b = 0;
 
@@ -330,7 +341,12 @@ void YFrameWindow::layoutButtons() {
     }
 
     if (titleButtonsRight) {
-        int xPos = width() - 2 * borderX();
+#ifdef CONFIG_LOOK_PIXMAP
+        int xPos(width() - 2 * borderX() - 
+		(titleQ[pi] ? titleQ[pi]->width() : 0));
+#else
+        int xPos(width() - 2 * borderX());
+#endif
 
         for (const char *bc = titleButtonsRight; *bc; bc++) {
             YFrameButton *b = 0;
