@@ -1,4 +1,3 @@
-
 #ifdef FONTS_ADOBE
 #define FONT(pt) "-b&h-lucida-medium-r-*-*-*-" #pt "-*-*-*-*-*-*"
 #define BOLDFONT(pt) "-b&h-lucida-bold-r-*-*-*-" #pt "-*-*-*-*-*-*"
@@ -12,7 +11,7 @@
 #endif
 
 #define CONFIG_DEFAULT_LOOK lookNice
-#define CONFIG_DEFAULT_THEME "warp3/default.theme"
+#define CONFIG_DEFAULT_THEME "Infadel2/default.theme"
 
 #ifdef CFGDEF
 #define XSV(t,a,b) t a = b;
@@ -89,6 +88,7 @@ XIV(bool, focusRootWindow             , false)
 XIV(bool, pointerColormap             , true)
 XIV(bool, sizeMaximized               , false)
 XIV(bool, showMoveSizeStatus          , true)
+XIV(bool, workspaceSwitchStatus       , true)
 XIV(bool, beepOnNewMail               , false)
 XIV(bool, warpPointer                 , false)
 XIV(bool, opaqueMove                  , true)
@@ -96,8 +96,10 @@ XIV(bool, opaqueResize                , true)
 #ifdef CONFIG_TASKBAR
 XIV(bool, showTaskBar                 , true)
 XIV(bool, taskBarAtTop                , false)
+XIV(bool, taskBarKeepBelow            , true)
 XIV(bool, taskBarShowClock            , true)
 XIV(bool, taskBarShowApm              , false)
+XIV(bool, taskBarShowApmTime          , true) // mschy
 XIV(bool, taskBarShowMailboxStatus    , true)
 XIV(bool, taskBarShowStartMenu        , true)
 XIV(bool, taskBarShowWindowListMenu   , true)
@@ -128,19 +130,31 @@ XIV(bool, strongPointerFocus          , false)
 XIV(bool, grabRootWindow              , true)
 XIV(bool, snapMove                    , true)
 XIV(bool, centerBackground            , false)
+XIV(bool, supportSemitransparency     , true)
 XIV(bool, edgeWorkspaceSwitching      , false)
 XIV(bool, showPopupsAbovePointer      , false)
 XIV(bool, replayMenuCancelClick       , false)
 XIV(bool, limitSize                   , true)
 XIV(bool, limitPosition               , true)
+XIV(bool, limitByDockLayer            , false)
+XIV(bool, considerHorizBorder         , false)
+XIV(bool, considerVertBorder          , false)
 XIV(bool, win95keys                   , false)
 XIV(bool, modMetaIsCtrlAlt            , true)
 XIV(bool, autoReloadMenus             , true)
 XIV(bool, showFrameIcon               , true)
-XIV(bool, autoDetectGnome             , true)
+XIV(bool, autoDetectGnome             , false)
 XIV(bool, clientMouseActions          , true)
 XIV(bool, titleBarCentered            , false)
 XIV(bool, showThemesMenu              , true)
+XIV(bool, showHelp                    , true)
+#ifdef CONFIG_IMLIB
+XIV(bool, gnomeFolderIcon             , true)
+#endif
+XIV(bool, gnomeAppsMenuAtToplevel     , false)
+XIV(bool, gnomeUserMenuAtToplevel     , false)
+XIV(bool, kdeMenuAtToplevel           , false)
+XIV(bool, showAddressBar              , true)
 XIV(bool, confirmLogout               , true)
 #ifdef I18N
 XIV(bool, multiByte                   , true)
@@ -163,7 +177,6 @@ XIV(unsigned int, ToolTipTime         , 5000)
 XIV(unsigned int, MenuActivateDelay   , 10)
 XIV(unsigned int, SubmenuActivateDelay, 300)
 XIV(unsigned int, EdgeResistance      , 32)
-XIV(unsigned int, useXButton          , 1)
 XIV(unsigned int, snapDistance        , 8)
 XIV(unsigned int, pointerFocusDelay   , 200);
 XIV(unsigned int, autoRaiseDelay      , 400)
@@ -173,6 +186,7 @@ XIV(unsigned int, scrollBarStartDelay , 500)
 XIV(unsigned int, scrollBarDelay      , 30)
 XIV(unsigned int, autoScrollStartDelay, 500)
 XIV(unsigned int, autoScrollDelay     , 60)
+XIV(unsigned int, workspaceStatusTime, 2500)
 XIV(unsigned int, useRootButtons      , 255) // bitmask=all
 XIV(unsigned int, buttonRaiseMask     , 1)
 XIV(unsigned int, rootWinMenuButton   , 1)
@@ -180,6 +194,7 @@ XIV(unsigned int, rootWinListButton   , 2)
 XIV(unsigned int, rootMenuButton      , 3)
 XIV(unsigned int, titleMaximizeButton , 1)
 XIV(unsigned int, titleRollupButton   , 2)
+XIV(unsigned int, msgBoxDefaultAction , 0)
 XIV(unsigned int, mailCheckDelay      , 30)
 XSV(const char *, titleButtonsLeft          , "s")
 XSV(const char *, titleButtonsRight         , "xmir")
@@ -206,7 +221,8 @@ XSV(const char *, apmFontName               , TTFONT(140))
 XSV(const char *, inputFontName             , TTFONT(140))
 XSV(const char *, iconPath                  , 0)
 XSV(const char *, libDir                    , LIBDIR)
-XSV(const char *, configDir                 , CONFIGDIR)
+XSV(const char *, configDir                 , CFGDIR)
+XSV(const char *, kdeDataDir                , KDEDIR)
 XSV(const char *, mailBoxPath               , 0)
 XSV(const char *, mailCommand               , 0)
 XSV(const char *, newMailCommand            , 0)
@@ -223,8 +239,13 @@ XSV(const char *, netDevice                 , "ppp0")
 XSV(const char *, cpuCommand                , 0)
 XSV(const char *, netCommand                , 0)
 XSV(const char *, addressBarCommand         , 0)
+#ifdef I18N
+XSV(const char *, fmtTime                   , "%X")
+XSV(const char *, fmtDate                   , "%c")
+#else
 XSV(const char *, fmtTime                   , "%H:%M:%S")
 XSV(const char *, fmtDate                   , "%B %A %Y-%m-%d %H:%M:%S %Z")
+#endif
 XSV(const char *, clrDialog                 , "rgb:C0/C0/C0")
 XSV(const char *, clrActiveBorder           , "rgb:C0/C0/C0")
 XSV(const char *, clrInactiveBorder         , "rgb:C0/C0/C0")
@@ -234,6 +255,8 @@ XSV(const char *, clrActiveTitleBar         , "rgb:00/00/A0")
 XSV(const char *, clrInactiveTitleBar       , "rgb:80/80/80")
 XSV(const char *, clrActiveTitleBarText     , "rgb:FF/FF/FF")
 XSV(const char *, clrInactiveTitleBarText   , "rgb:00/00/00")
+XSV(const char *, clrActiveTitleBarShadow   , "")
+XSV(const char *, clrInactiveTitleBarShadow , "")
 XSV(const char *, clrNormalMinimizedWindow  , "rgb:C0/C0/C0")
 XSV(const char *, clrNormalMinimizedWindowText, "rgb:00/00/00")
 XSV(const char *, clrActiveMinimizedWindow  , "rgb:E0/E0/E0")
@@ -330,8 +353,12 @@ static struct {
     OBV("PointerColormap", &pointerColormap, "Colormap focus follows pointer"), //
     OBV("LimitSize", &limitSize, "Limit initial size of windows to screen"), //
     OBV("LimitPosition", &limitPosition, "Limit initial position of windows to screen"), //
+    OBV("LimitByDockLayer", &limitByDockLayer, "Let the Dock layer limit the workspace (incompatible with GNOME Panel)"), //
+    OBV("ConsiderHBorder", &considerHorizBorder, "Consider border frames when maximizing horizontally"), //
+    OBV("ConsiderVBorder", &considerVertBorder, "Consider border frames when maximizing vertically"), //
     OBV("SizeMaximized", &sizeMaximized, "Maximized windows can be resized"), //
     OBV("ShowMoveSizeStatus", &showMoveSizeStatus, "Show position status window during move/resize"), //
+    OBV("ShowWorkspaceStatus", &workspaceSwitchStatus, "Show name of current workspace while switching"), //
     OBV("MinimizeToDesktop", &minimizeToDesktop, "Display mini-icons on desktop for minimized windows"), //
     OBV("StrongPointerFocus", &strongPointerFocus, "Always maintain focus under mouse window (makes some keyboard support non-functional or unreliable"),
     OBV("OpaqueMove", &opaqueMove, "Opaque window move"), //
@@ -355,15 +382,18 @@ static struct {
     OBV("SnapMove", &snapMove, "Snap to nearest screen edge/window when moving windows"), //
     OBV("EdgeSwitch", &edgeWorkspaceSwitching, "Workspace switches by moving mouse to left/right screen edge"), //
     OBV("DesktopBackgroundCenter", &centerBackground, "Display desktop background centered and not tiled"),
+    OBV("SupportSemitransparency", &supportSemitransparency, "Support for semitransparent terminals like Eterm or gnome-terminal"), //
     OBV("AutoReloadMenus", &autoReloadMenus, "Reload menu files automatically"),
     OBV("ShowMenuButtonIcon", &showFrameIcon, "Show application icon over menu button"),
     OBV("AutoDetectGNOME", &autoDetectGnome, "Automatically disable some functionality when running under GNOME."),
 #ifdef CONFIG_TASKBAR
     OBV("ShowTaskBar", &showTaskBar, "Show task bar"), //
     OBV("TaskBarAtTop" , &taskBarAtTop, "Task bar at top of the screen"), //
+    OBV("TaskBarKeepBelow" , &taskBarKeepBelow, "Keep the task bar below regular windows"), //
     OBV("TaskBarAutoHide", &taskBarAutoHide, "Auto hide task bar after delay"),//
     OBV("TaskBarShowClock", &taskBarShowClock, "Show clock on task bar"), //
     OBV("TaskBarShowAPMStatus", &taskBarShowApm, "Show APM status on task bar"),
+    OBV("TaskBarShowAPMTime", &taskBarShowApmTime, "Show APM status on task bar in time-format"), // mschy
     OBV("TaskBarClockLeds", &prettyClock, "Task bar clock uses nice pixmapped LCD display"), //
     OBV("TaskBarShowMailboxStatus", &taskBarShowMailboxStatus, "Show mailbox status on task bar"), //
     OBV("TaskBarMailboxStatusBeepOnNewMail", &beepOnNewMail, "Beep when new mail arrives"), //
@@ -381,8 +411,16 @@ static struct {
     OBV("ClientWindowMouseActions", &clientMouseActions, "Allow mouse actions on client windows (buggy with some programs)"),
     OBV("TitleBarCentered", &titleBarCentered, "Draw window title centered"),
     OBV("ShowThemesMenu", &showThemesMenu, "Show themes submenu"),
+    OBV("ShowHelp", &showHelp, "Show the help menu item"),
+#ifdef CONFIG_IMLIB
+    OBV("GNOMEFolderIcon", &gnomeFolderIcon, "Show GNOME's folder icon in GNOME menus"),
+#endif
+    OBV("GNOMEAppsMenuAtToplevel", &gnomeAppsMenuAtToplevel, "Create GNOME application menu at toplevel"),
+    OBV("GNOMEUserMenuAtToplevel", &gnomeUserMenuAtToplevel, "Create GNOME user menu at toplevel"),
+    OBV("KDEMenuAtToplevel", &kdeMenuAtToplevel, "Create KDE menu at toplevel"),
+    OBV("ShowAddressBar", &showAddressBar, "Show address bar in task bar"),
 #ifdef I18N
-    OBV("MultiByte", &multiByte, "Multibyte I18N support"),
+    OBV("MultiByte", &multiByte, "Overrides automatic multiple byte detection"),
 #endif
     OBV("ConfirmLogout", &confirmLogout, "Confirm logout")
 };
@@ -409,7 +447,7 @@ static struct {
     OIV("SubmenuMenuActivateDelay", &SubmenuActivateDelay, 0, 5000, "Delay before activating menu submenus"), //
 #ifndef LITE
     OIV("ToolTipDelay", &ToolTipDelay, 0, 5000, "Delay before tooltip window is displayed"), //
-    OIV("ToolTipTime", &ToolTipTime, 0, 60000, "Time before tooltip window is hidden"), //
+    OIV("ToolTipTime", &ToolTipTime, 0, 60000, "Time before tooltip window is hidden (0 means never"), //
 #endif
     OIV("AutoHideDelay", &autoHideDelay, 0, 5000, "Delay before task bar is automatically hidden"), //
     OIV("AutoRaiseDelay", &autoRaiseDelay, 0, 5000, "Delay before windows are auto raised"), //
@@ -421,6 +459,7 @@ static struct {
     OIV("ScrollBarDelay", &scrollBarDelay, 0, 5000, "Scroll bar autoscroll delay"), //
     OIV("AutoScrollStartDelay", &autoScrollStartDelay, 0, 5000, "Auto scroll start delay"), //
     OIV("AutoScrollDelay", &autoScrollDelay, 0, 5000, "Auto scroll delay"), //
+    OIV("WorkspaceStatusTime", &workspaceStatusTime, 0, 2500, "Time before workspace status window is hidden"), //
     OIV("UseRootButtons", &useRootButtons, 0, 255, "Bitmask of root window button click to use in window manager"), //
     OIV("ButtonRaiseMask", &buttonRaiseMask, 0, 255, "Bitmask of buttons that raise the window when pressed"), //
     OIV("DesktopWinMenuButton", &rootWinMenuButton, 0, 20, "Desktop mouse-button click to show the menu"),
@@ -428,6 +467,7 @@ static struct {
     OIV("DesktopMenuButton", &rootMenuButton, 0, 20, "Desktop mouse-button click to show the window list menu"),
     OIV("TitleBarMaximizeButton", &titleMaximizeButton, 0, 5, "TitleBar mouse-button double click to maximize the window"),
     OIV("TitleBarRollupButton", &titleRollupButton, 0, 5, "TitleBar mouse-button double clock to rollup the window"),
+    OIV("MsgBoxDefaultAction", &msgBoxDefaultAction, 0, 1, "Preselect to Cancel (0) or the OK (1) button in message boxes"),
     OIV("MailCheckDelay", &mailCheckDelay, 0, (3600*24), "Delay between new-mail checks. (seconds)"),
 #ifdef CONFIG_TASKBAR
     OIV("TaskBarCPUSamples", &taskBarCPUSamples, 2, 1000, "Width of CPU Monitor")
@@ -447,6 +487,7 @@ static struct {
     OSV("TitleButtonsRight", &titleButtonsRight, "Titlebar buttons from right to left (x=close, m=max, i=min, h=hide, r=rollup, s=sysmenu, d=depth)"),
     OSV("TitleButtonsSupported", &titleButtonsSupported, "Titlebar buttons supported by theme (x,m,i,r,h,s,d)"),
     OSV("IconPath", &iconPath, "Icon search path (colon separated)"), //
+    OSV("KDEDataDir", &kdeDataDir, "Root directory for KDE data"), //
     OSV("MailBoxPath", &mailBoxPath, "Mailbox path (use $MAIL instead)"),
     OSV("MailCommand", &mailCommand, "Command to run on mailbox"), //
     OSV("NewMailCommand", &newMailCommand, "Command to run when new mail arrives"), //
@@ -483,6 +524,7 @@ static struct {
     OSV("ToolTipFontName", &toolTipFontName, ""), //
     OSV("ClockFontName", &clockFontName, ""), //
     OSV("ApmFontName", &apmFontName, ""), //
+    OSV("InputFontName", &inputFontName, ""),
     OSV("LabelFontName", &labelFontName, ""), //
     OSV("ColorDialog", &clrDialog, ""),
     OSV("ColorActiveBorder", &clrActiveBorder, ""),
@@ -497,6 +539,8 @@ static struct {
     OSV("ColorNormalTitleBar", &clrInactiveTitleBar, ""),
     OSV("ColorActiveTitleBarText", &clrActiveTitleBarText, ""),
     OSV("ColorNormalTitleBarText", &clrInactiveTitleBarText, ""),
+    OSV("ColorActiveTitleBarShadow", &clrActiveTitleBarShadow, ""),
+    OSV("ColorNormalTitleBarShadow", &clrInactiveTitleBarShadow, ""),
     OSV("ColorNormalMinimizedWindow", &clrNormalMinimizedWindow, ""),
     OSV("ColorNormalMinimizedWindowText", &clrNormalMinimizedWindowText, ""),
     OSV("ColorActiveMinimizedWindow", &clrActiveMinimizedWindow, ""),
@@ -588,8 +632,10 @@ static struct {
     OKV("KeySysAddressBar", gKeySysAddressBar, ""),
     OKV("KeySysWorkspacePrev", gKeySysWorkspacePrev, ""),
     OKV("KeySysWorkspaceNext", gKeySysWorkspaceNext, ""),
+    OKV("KeySysWorkspaceLast", gKeySysWorkspaceLast, ""),
     OKV("KeySysWorkspacePrevTakeWin", gKeySysWorkspacePrevTakeWin, ""),
     OKV("KeySysWorkspaceNextTakeWin", gKeySysWorkspaceNextTakeWin, ""),
+    OKV("KeySysWorkspaceLastTakeWin", gKeySysWorkspaceLastTakeWin, ""),
     OKV("KeySysWorkspace1", gKeySysWorkspace1, ""),
     OKV("KeySysWorkspace2", gKeySysWorkspace2, ""),
     OKV("KeySysWorkspace3", gKeySysWorkspace3, ""),
