@@ -2,6 +2,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <X11/Xlocale.h>
 #include "config.h"
 #include "ylib.h"
 #include <X11/Xatom.h>
@@ -12,15 +13,20 @@
 #include "sysdep.h"
 #include "yaction.h"
 #include "ymenuitem.h"
+#include "prefs.h"
 
 //#define DUMP
 //#define TEXT
 
 #include <unistd.h>
 
+#include "intl.h"
+
 #define LINE(c) ((c) == '\r' || (c) == '\n')
 #define SPACE(c) ((c) == ' ' || (c) == '\t' || LINE(c))
 
+char const * YApplication::Name = "icehelp";
+
 class HTListener {
 public:
     virtual void activateURL(const char *url) = 0;
@@ -31,7 +37,7 @@ public:
     text_node(const char *t, int l, int _x, int _y, int _w, int _h) {
         text = t; len = l; next = 0; x = _x; y = _y; w = _w; h = _h;
     }
-
+    
     int x, y;
     short w, h, len; // int?
     const char *text;
@@ -97,7 +103,7 @@ node *add(node **first, node *last, node *n) {
 }
 
 void add_attribute(node *n, char *abuf, char *vbuf) {
-    //printf("[%s]=[%s]\n", abuf, vbuf ? vbuf : "");
+    //msg("[%s]=[%s]", abuf, vbuf ? vbuf : "");
 
     n->attr = (attribute *)realloc(n->attr, sizeof(attribute) * (n->nattr + 1));
     assert(n->attr != 0);
@@ -494,9 +500,10 @@ class HTextView: public YWindow,
     public YScrollBarListener, public YScrollable, public YActionListener
 {
 public:
-    HTextView(HTListener *fL, YScrollView *v, YWindow *parent): YWindow(parent), listener(fL) {
+    HTextView(HTListener *fL, YScrollView *v, YWindow *parent):
+	YWindow(parent), view(v), listener(fL) {
         view = v;
-        fVerticalScroll = view->getVerticalScrollBar();;
+        fVerticalScroll = view->getVerticalScrollBar();
         fVerticalScroll->setScrollBarListener(this);
         fHorizontalScroll = view->getHorizontalScrollBar();
         fHorizontalScroll->setScrollBarListener(this);
@@ -529,16 +536,16 @@ public:
 
         menu = new YMenu();
         menu->setActionListener(this);
-        menu->addItem("Back", 0, "Alt+Left", actionNone)->setEnabled(false);
-        menu->addItem("Forward", 0, "Alt+Right", actionNone)->setEnabled(false);
+        menu->addItem(_("Back"), 0, _("Alt+Left"), actionNone)->setEnabled(false);
+        menu->addItem(_("Forward"), 0, _("Alt+Right"), actionNone)->setEnabled(false);
         menu->addSeparator();
-        prevItem = menu->addItem("Previous", 0, "", actionPrev);
-        nextItem = menu->addItem("Next", 0, "", actionNext);
+        prevItem = menu->addItem(_("Previous"), 0, "", actionPrev);
+        nextItem = menu->addItem(_("Next"), 0, "", actionNext);
         menu->addSeparator();
-        contentsItem = menu->addItem("Contents", 0, "", actionContents);
-        menu->addItem("Index", 0, "", actionNone)->setEnabled(false);
+        contentsItem = menu->addItem(_("Contents"), 0, "", actionContents);
+        menu->addItem(_("Index"), 0, "", actionNone)->setEnabled(false);
         menu->addSeparator();
-        menu->addItem("Close", 0, "Ctrl+Q", actionClose);
+        menu->addItem(_("Close"), 0, _("Ctrl+Q"), actionClose);
     }
 
     ~HTextView() {
@@ -728,12 +735,12 @@ void HTextView::layout() {
 
 void addState(int &state, int value) {
     state |= value;
-    //printf("addState=%d %d\n", state, value);
+    //msg("addState=%d %d", state, value);
 }
 
 void removeState(int &state, int value) {
     state &= ~value;
-    //printf("removeState=%d %d\n", state, value);
+    //msg("removeState=%d %d", state, value);
 }
 
 void HTextView::par(int &state, int &x, int &y, int &h, const int left) {
@@ -844,7 +851,7 @@ void HTextView::layout(node *parent, node *n1, int left, int right, int &x, int
                                 b = c;
                             } else
                                 break;
-                            ///printf("width=%d %d / %d / %d\n", wc, x, left, right);
+                            ///msg("width=%d %d / %d / %d", wc, x, left, right);
                         } while (*c);
                     }
 
@@ -854,10 +861,10 @@ void HTextView::layout(node *parent, node *n1, int left, int right, int &x, int
                         {
                             char *f = b;
                             int ll;
-                            printf("x=%d left=%d line: ", x, left);
+                            msg("x=%d left=%d line: ", x, left);
                             ll = 0;
                             while (f < c) { putchar(*f); f++; ll++; }
-                            printf("[len=%d]\n", ll);
+                            msg("[len=%d]", ll);
                         }
 #endif
                         par(state, x, y, h, left);
@@ -880,7 +887,7 @@ void HTextView::layout(node *parent, node *n1, int left, int right, int &x, int
                             }
                         }
                     }
-                    //printf("len=%d x=%d left=%d %s\n", c - b, x, left, b);
+                    //msg("len=%d x=%d left=%d %s", c - b, x, left, b);
                 }
             }
             break;
@@ -1018,7 +1025,7 @@ void HTextView::layout(node *parent, node *n1, int left, int right, int &x, int
         case node::li:
             //puts("<LI>");
 
-            //printf("state=%d\n", state);
+            //msg("state=%d", state);
             epar(state, x, y, h, left);
             //if (state & sfText)
             y = h;
@@ -1046,7 +1053,7 @@ void HTextView::layout(node *parent, node *n1, int left, int right, int &x, int
             x = left;
             removeState(state, sfPar);
             addState(state, sfText);
-            //printf("set=%d\n", state);
+            //msg("set=%d", state);
             break;
         default:
             if (n->container)
@@ -1125,7 +1132,9 @@ public:
         view->show();
         scroll->show();
 
-        XStoreName(app->display(), handle(), fPath);
+        setTitle(fPath);
+	setClassHint("browser", "IceHelp");
+
         file = getIcon("file");
         Pixmap icons[4];
         icons[0] = file->small()->pixmap();
@@ -1148,7 +1157,7 @@ public:
         char link[1024];
         char *r;
 
-        //printf("link: %s\n", url);
+        //msg("link: %s", url);
 
         strcpy(link, fPath);
 
@@ -1216,6 +1225,12 @@ void HTextView::handleClick(const XButtonEvent &up, int /*count*/) {
 }
 
 int main(int argc, char **argv) {
+#ifdef I18N
+    char *loc = setlocale(LC_ALL, "");
+    if (loc == NULL || !strcmp(loc, "C") || !strcmp(loc, "POSIX"))
+        multiByte = false;
+    else multiByte = true;
+#endif
     YApplication app(&argc, &argv);
 
     if (argc > 1) {
@@ -1229,7 +1244,19 @@ int main(int argc, char **argv) {
 void FileView::loadFile() {
     FILE *fp = fopen(fPath, "r");
     if (fp == 0) {
-        printf("invalid path: %s\n", fPath);
+        warn(_("Invalid path: %s\n"), fPath);
+	root = new node(node::div);
+	node * last(NULL);
+
+	node * txt(new node(node::text));
+	txt->txt = _("Invalid path: ");
+	last = add(&root, NULL, txt);
+
+	txt = new node(node::text);
+	txt->txt = fPath;
+	last = add(&root, last, txt);
+
+        view->setData(root);
         return ;
     }
     if (fp) {
@@ -1274,7 +1301,7 @@ int width = 80;
 
 void clear_line() {
     if (pos != 0)
-        printf("\n");
+        puts("");
     pos = 0;
 }
 
@@ -1348,7 +1375,7 @@ void wrap(int left, char *text) {
             text++;
         if (*text) {
             //if (pos != width)
-                printf("\n");
+                puts("");
             pos = 0;
             tab(left);
         }
@@ -1440,7 +1467,7 @@ void dump(int flags, int left, node *n, node *up) {
             break;
         case node::paragraph:
             clear_line();
-            new_line(1);;
+            new_line(1);
             break;
         case node::line:
             new_line(1);
