@@ -13,6 +13,8 @@ extern "C" {
 #include <sys/mman.h>
 }
 
+#include "intl.h"
+
 YIcon *file = 0;
 
 extern Atom _XA_WIN_ICONS;
@@ -58,13 +60,13 @@ public:
         actionToggleHexView = new YAction();
         menu = new YMenu();
         menu->setActionListener(this);
-        //menu->addItem("Find...", 0, "Ctrl+F", actionFind);
+        //menu->addItem(_("Find..."), 0, _("Ctrl+F"), actionFind);
         menu->addSeparator();
-        menu->addItem("Hex View", 0, "Ctrl+H", actionToggleHexView);
-        menu->addItem("Expand Tabs", 0, "Ctrl+T", actionToggleExpandTabs);
-        menu->addItem("Wrap Lines", 0, "Ctrl+W", actionToggleWrapLines);
+        menu->addItem(_("Hex View"), 0, _("Ctrl+H"), actionToggleHexView);
+        menu->addItem(_("Expand Tabs"), 0, _("Ctrl+T"), actionToggleExpandTabs);
+        menu->addItem(_("Wrap Lines"), 0, _("Ctrl+W"), actionToggleWrapLines);
         menu->addSeparator();
-        menu->addItem("Close", 0, "Ctrl+Q", actionClose);
+        menu->addItem(_("Close"), 0, _("Ctrl+Q"), actionClose);
     }
 
     ~TextView() {
@@ -328,7 +330,7 @@ public:
         if ((ty + wy + wheight) % fontHeight)
             l2++;
         int y = l1 * fontHeight - ty;
-        //printf("l1=%d,l2=%d\n", l1, l2);
+        //msg("l1=%d,l2=%d", l1, l2);
         for (int l = l1; l < l2; l++) {
             if (hexView) {
                 if (l >= chunkCount)
@@ -518,7 +520,7 @@ public:
         view->show();
         scroll->show();
 
-        XStoreName(app->display(), handle(), fPath);
+        setTitle(fPath);
         file = getIcon("file");
         Pixmap icons[4];
         icons[0] = file->small()->pixmap();
@@ -594,6 +596,12 @@ private:
 };
 
 int main(int argc, char **argv) {
+
+#ifdef ENABLE_NLS
+    bindtextdomain(PACKAGE, LOCDIR);
+    textdomain(PACKAGE);
+#endif
+
     YApplication app(&argc, &argv);
 
     if (argc > 1) {
