@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 
@@ -17,6 +17,8 @@
 #include "prefs.h"
 #include <dirent.h>
 
+#include "intl.h"
+
 ThemesMenu::ThemesMenu(YWindow *parent): ObjectMenu(parent) {
     const char *homeDir = getenv("HOME");
     char *path;
@@ -52,14 +54,15 @@ YMenuItem *ThemesMenu::findTheme(YMenu *container, const char *themeName) {
 void ThemesMenu::addTheme(YMenu *container, const char *name, const char *name_theme) {
     char **args = (char **)MALLOC(6 * sizeof(char *));
     if (args) {
-        args[0] = newstr("icewm"EXEEXT);
+        args[0] = newstr(ICEWMEXE);
         args[1] = newstr("-t");
         args[2] = newstr(name_theme);
-        args[3] = configArg ? (char *)"-c" : 0;
-        args[4] = configArg;
+        args[3] = configArg ? newstr("-c") : 0;
+        args[4] = newstr(configArg);
         args[5] = 0;
         if (args[0] && args[1] && args[2]) {
-            DProgram *re_theme = DProgram::newProgram(name, 0, true, "icewm"EXEEXT, args);
+            DProgram *re_theme =
+		DProgram::newProgram(name, 0, true, 0, ICEWMEXE, args);
             if (re_theme) {
                 YMenuItem *item = container->add(new DObjectMenuItem(re_theme));
                 if (themeName && strcmp(themeName, name_theme) == 0)
@@ -75,6 +78,7 @@ void ThemesMenu::findThemes(const char *path, YMenu *container) {
     char *npath = 0;
     char *dpath = 0;
     int dplen = strlen(path);
+    bool first = true;
 
     if (dplen == 0 || path[dplen - 1] != '/') {
         npath = strJoin(path, "/", NULL);
@@ -90,8 +94,16 @@ void ThemesMenu::findThemes(const char *path, YMenu *container) {
             YMenuItem *im = findTheme(container, de->d_name);
             if (im == 0) {
                 npath = strJoin(dpath, de->d_name, tname, NULL);
-                if (npath && access(npath, R_OK) == 0)
+                if (npath && access(npath, R_OK) == 0) {
+		    if (first) {
+			first = false;
+			if (itemCount())
+			    addSeparator();
+			addLabel(path);
+			addSeparator();
+		    }
                     addTheme(container, de->d_name, npath + dplen);
+		}
                 delete [] npath;
             }
             char *subdir = strJoin(dpath, de->d_name, NULL);
