@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 #include "yfull.h"
@@ -33,6 +33,10 @@
 #include <X11/Xlocale.h>
 #endif
 
+#include "intl.h"
+
+char const * YApplication::Name = "icewm";
+
 int initializing = 1;
 int rebootOrShutdown = 0;
 
@@ -41,16 +45,20 @@ YWindowManager *manager = 0;
 
 char *keysFile = 0;
 
-Atom XA_IcewmWinOptHint;
+Atom XA_IcewmWinOptHint(None);
+Atom XA_ICEWM_FONT_PATH(None);
 
-Cursor sizeRightPointer;
-Cursor sizeTopRightPointer;
-Cursor sizeTopPointer;
-Cursor sizeTopLeftPointer;
-Cursor sizeLeftPointer;
-Cursor sizeBottomLeftPointer;
-Cursor sizeBottomPointer;
-Cursor sizeBottomRightPointer;
+Atom _XA_XROOTPMAP_ID(None);
+Atom _XA_XROOTCOLOR_PIXEL(None);
+
+YCursor YWMApp::sizeRightPointer;
+YCursor YWMApp::sizeTopRightPointer;
+YCursor YWMApp::sizeTopPointer;
+YCursor YWMApp::sizeTopLeftPointer;
+YCursor YWMApp::sizeLeftPointer;
+YCursor YWMApp::sizeBottomLeftPointer;
+YCursor YWMApp::sizeBottomPointer;
+YCursor YWMApp::sizeBottomRightPointer;
 
 YMenu *windowMenu = 0;
 YMenu *moveMenu = 0;
@@ -69,25 +77,23 @@ char *configArg = 0;
 PhaseType phase = phaseStartup;
 
 static void registerProtocols() {
-    Atom win_proto[10];
-    unsigned int i = 0;
-
-    win_proto[i++] = _XA_WIN_WORKSPACE;
-    win_proto[i++] = _XA_WIN_WORKSPACE_COUNT;
-    win_proto[i++] = _XA_WIN_WORKSPACE_NAMES;
-    win_proto[i++] = _XA_WIN_ICONS;
-    win_proto[i++] = _XA_WIN_WORKAREA;
-
-    win_proto[i++] = _XA_WIN_STATE;
-    win_proto[i++] = _XA_WIN_HINTS;
-    win_proto[i++] = _XA_WIN_LAYER;
-    win_proto[i++] = _XA_WIN_SUPPORTING_WM_CHECK;
-    win_proto[i++] = _XA_WIN_CLIENT_LIST;
+    Atom win_proto[] = {
+	_XA_WIN_WORKSPACE,
+	_XA_WIN_WORKSPACE_COUNT,
+	_XA_WIN_WORKSPACE_NAMES,
+	_XA_WIN_ICONS,
+	_XA_WIN_WORKAREA,
+	
+	_XA_WIN_STATE,
+	_XA_WIN_HINTS,
+	_XA_WIN_LAYER,
+	_XA_WIN_SUPPORTING_WM_CHECK,
+	_XA_WIN_CLIENT_LIST
+    };
 
     XChangeProperty(app->display(), manager->handle(),
-                    _XA_WIN_PROTOCOLS, XA_ATOM, 32,
-                    PropModeReplace, (unsigned char *)win_proto, i);
-
+                    _XA_WIN_PROTOCOLS, XA_ATOM, 32, PropModeReplace,
+		    (unsigned char *)win_proto, ACOUNT(win_proto));
 
     YWindow *checkWindow = new YWindow();
     Window xid = checkWindow->handle();
@@ -115,7 +121,17 @@ static void unregisterProtocols() {
     XDeleteProperty(app->display(),
                     manager->handle(),
                     _XA_WIN_PROTOCOLS);
+		    
+    if (supportSemitransparency) {
+        XDeleteProperty(app->display(),
+			manager->handle(),
+			_XA_XROOTPMAP_ID);
+        XDeleteProperty(app->display(),
+			manager->handle(),
+			_XA_XROOTCOLOR_PIXEL);
+    }
 }
+
 static void initIconSize() {
     XIconSize *is;
 
@@ -135,17 +151,98 @@ static void initIconSize() {
 
 static void initAtoms() {
     XA_IcewmWinOptHint = XInternAtom(app->display(), "_ICEWM_WINOPTHINT", False);
+    XA_ICEWM_FONT_PATH = XInternAtom(app->display(), "ICEWM_FONT_PATH", False);
+    _XA_XROOTPMAP_ID = XInternAtom(app->display(), "_XROOTPMAP_ID", False);
+    _XA_XROOTCOLOR_PIXEL = XInternAtom(app->display(), "_XROOTCOLOR_PIXEL", False);
+}
+
+static void initFontPath() {
+#ifndef LITE
+    if (themeName) { // =================== find the current theme directory ===
+	char themeSubdir[PATH_MAX];
+	strncpy(themeSubdir, themeName, sizeof(themeSubdir) - 1);
+	themeSubdir[sizeof(themeSubdir) - 1] = '\0';
+
+	char * strfn(strrchr(themeSubdir, '/'));
+	if (strfn) *strfn = '\0';
+
+	// ================================ is there a file named fonts.dir? ===  
+	strfn = strJoin("themes/", themeSubdir, "/fonts.dir", NULL);
+	char * fontsDir(app->findConfigFile(strfn));
+	delete[] strfn;
+
+	if (fontsDir) { // =========================== build a new font path ===
+	    strfn = strrchr(fontsDir, '/');
+	    if (strfn) *strfn = '\0';
+
+	    int ndirs; // ------------------- retrieve the old X's font path ---
+	    char ** fontPath(XGetFontPath(app->display(), &ndirs));
+
+	    char * newFontPath[ndirs + 1];
+	    newFontPath[0] = fontsDir;
+
+	    if (fontPath)
+		memcpy(newFontPath + 1, fontPath, ndirs * sizeof (char *));
+	    else
+		warn(_("Unable to get current font path."));
+
+#ifdef DEBUG
+	    for (int n = 0; n < ndirs + 1; ++n)
+		MSG(("Font path element %d: %s", n, newFontPath[n]));
+#endif
+
+	    char * icewmFontPath; // ---------- find death icewm's font path ---
+	    Atom r_type; int r_format;
+	    unsigned long count, bytes_remain;
+
+	    if (XGetWindowProperty(app->display(),
+				   manager->handle(),
+				   XA_ICEWM_FONT_PATH,
+				   0, PATH_MAX, False, XA_STRING,
+				   &r_type, &r_format,
+				   &count, &bytes_remain, 
+				   (unsigned char **) &icewmFontPath) ==
+				   Success && icewmFontPath) {
+		if (r_type == XA_STRING && r_format == 8) {
+		    for (int n(ndirs); n > 0; --n) // ---- remove death paths ---
+			if (!strcmp(icewmFontPath, newFontPath[n])) {
+			    if (n != ndirs)
+				memmove(newFontPath + n, newFontPath + n + 1,
+					(ndirs - n) * sizeof(char *));
+			    --ndirs;
+			}
+		} else
+		    warn(_("Unexpected format of ICEWM_FONT_PATH property"));
+
+		XFree(icewmFontPath);
+	    }
+
+#ifdef DEBUG
+	    for (int n = 0; n < ndirs + 1; ++n)
+		MSG(("Font path element %d: %s", n, newFontPath[n]));
+#endif
+	    // ----------------------------------------- set the new font path ---
+	    XChangeProperty(app->display(), manager->handle(),
+			    XA_ICEWM_FONT_PATH, XA_STRING, 8, PropModeReplace,
+			    (unsigned char *) fontsDir, strlen(fontsDir));
+	    XSetFontPath(app->display(), newFontPath, ndirs + 1);
+
+	    if (fontPath) XFreeFontPath(fontPath);
+	    delete[] fontsDir;
+	}
+    }
+#endif    
 }
 
 static void initPointers() {
-    sizeRightPointer = XCreateFontCursor(app->display(), XC_right_side);
-    sizeTopRightPointer = XCreateFontCursor(app->display(), XC_top_right_corner);
-    sizeTopPointer = XCreateFontCursor(app->display(), XC_top_side);
-    sizeTopLeftPointer = XCreateFontCursor(app->display(), XC_top_left_corner);
-    sizeLeftPointer = XCreateFontCursor(app->display(), XC_left_side);
-    sizeBottomLeftPointer = XCreateFontCursor(app->display(), XC_bottom_left_corner);
-    sizeBottomPointer = XCreateFontCursor(app->display(), XC_bottom_side);
-    sizeBottomRightPointer = XCreateFontCursor(app->display(), XC_bottom_right_corner);
+    YWMApp::sizeRightPointer.	   load("sizeR.xpm",  XC_right_side);
+    YWMApp::sizeTopRightPointer.   load("sizeTR.xpm", XC_top_right_corner);
+    YWMApp::sizeTopPointer.	   load("sizeT.xpm",  XC_top_side);
+    YWMApp::sizeTopLeftPointer.	   load("sizeTL.xpm", XC_top_left_corner);
+    YWMApp::sizeLeftPointer.	   load("sizeL.xpm",  XC_left_side);
+    YWMApp::sizeBottomLeftPointer. load("sizeBL.xpm", XC_bottom_left_corner);
+    YWMApp::sizeBottomPointer.	   load("sizeB.xpm",  XC_bottom_side);
+    YWMApp::sizeBottomRightPointer.load("sizeBR.xpm", XC_bottom_right_corner);
 }
 
 void replicatePixmap(YPixmap **pixmap, bool horiz) {
@@ -254,6 +351,7 @@ static void initPixmaps() {
         loadPixmap(tpaths, 0, "dframeABR.xpm", &frameBR[1][1]);
 
 
+        loadPixmap(tpaths, 0, "titleIJ.xpm", &titleJ[0]);
         loadPixmap(tpaths, 0, "titleIL.xpm", &titleL[0]);
         loadPixmap(tpaths, 0, "titleIS.xpm", &titleS[0]);
         loadPixmap(tpaths, 0, "titleIP.xpm", &titleP[0]);
@@ -261,6 +359,8 @@ static void initPixmaps() {
         loadPixmap(tpaths, 0, "titleIM.xpm", &titleM[0]);
         loadPixmap(tpaths, 0, "titleIB.xpm", &titleB[0]);
         loadPixmap(tpaths, 0, "titleIR.xpm", &titleR[0]);
+        loadPixmap(tpaths, 0, "titleIQ.xpm", &titleQ[0]);
+        loadPixmap(tpaths, 0, "titleAJ.xpm", &titleJ[1]);
         loadPixmap(tpaths, 0, "titleAL.xpm", &titleL[1]);
         loadPixmap(tpaths, 0, "titleAS.xpm", &titleS[1]);
         loadPixmap(tpaths, 0, "titleAP.xpm", &titleP[1]);
@@ -268,6 +368,7 @@ static void initPixmaps() {
         loadPixmap(tpaths, 0, "titleAM.xpm", &titleM[1]);
         loadPixmap(tpaths, 0, "titleAB.xpm", &titleB[1]);
         loadPixmap(tpaths, 0, "titleAR.xpm", &titleR[1]);
+        loadPixmap(tpaths, 0, "titleAQ.xpm", &titleQ[1]);
 
         for (int a = 0; a <= 1; a++) {
             for (int b = 0; b <= 1; b++) {
@@ -299,6 +400,21 @@ static void initPixmaps() {
     loadPixmap(tpaths, 0, "switchbg.xpm", &switchbackPixmap);
     loadPixmap(tpaths, 0, "logoutbg.xpm", &logoutPixmap);
 
+#ifdef CONFIG_TASKBAR
+    if (!showTaskBar) {
+        loadPixmap(tpaths, "taskbar/",
+                  "taskbuttonbg.xpm", &taskbuttonPixmap);
+        loadPixmap(tpaths, "taskbar/",
+                  "taskbuttonactive.xpm", &taskbuttonactivePixmap);
+    }
+#endif
+
+    YColor *c = DesktopBackgroundColor && DesktopBackgroundColor[0]
+	      ? new YColor(DesktopBackgroundColor) : YColor::black;
+    unsigned long bPixel = c->pixel();
+    Pixmap bgPixmap = None;
+    bool handleBackground = false;
+
     if (DesktopBackgroundPixmap && DesktopBackgroundPixmap[0]) {
         YPixmap *bg = 0;
         if (DesktopBackgroundPixmap[0] == '/') {
@@ -312,29 +428,34 @@ static void initPixmaps() {
             if (centerBackground) {
                 YPixmap *back = new YPixmap(desktop->width(), desktop->height());;
                 Graphics g(back);;
-                YColor *c = 0;
-                if (DesktopBackgroundColor && DesktopBackgroundColor[0])
-                    c = new YColor(DesktopBackgroundColor);
-                else
-                    c = YColor::black;
 
                 g.setColor(c);
                 g.fillRect(0, 0, desktop->width(), desktop->height());
-                g.drawPixmap(bg,
-                             (desktop->width() -  bg->width()) / 2,
-                             (desktop->height() - bg->height()) / 2);
+                g.drawPixmap(bg, (desktop->width() -  bg->width()) / 2,
+				(desktop->height() - bg->height()) / 2);
                 delete bg;
                 bg = back;
             }
-            XSetWindowBackgroundPixmap(app->display(), desktop->handle(), bg->pixmap());
-            XClearWindow(app->display(), desktop->handle());
+	    
+            XSetWindowBackgroundPixmap (app->display(), desktop->handle(),
+	    				bgPixmap = bg->pixmap());
+	    handleBackground = true;
         }
     } else if (DesktopBackgroundColor && DesktopBackgroundColor[0]) {
-        YColor *c = new YColor(DesktopBackgroundColor); //!!! leaks
-        XSetWindowBackground(app->display(), desktop->handle(), c->pixel());
-        XClearWindow(app->display(), desktop->handle());
+        XSetWindowBackground(app->display(), desktop->handle(), bPixel);
+	handleBackground = true;
     }
 
+    if (handleBackground && supportSemitransparency && 
+	_XA_XROOTPMAP_ID && _XA_XROOTCOLOR_PIXEL) {
+	XChangeProperty(app->display(), desktop->handle(),
+			_XA_XROOTPMAP_ID, XA_PIXMAP, 32,
+			PropModeReplace, (unsigned char*) &bgPixmap, 1);
+	XChangeProperty(app->display(), desktop->handle(),
+			_XA_XROOTCOLOR_PIXEL, XA_CARDINAL, 32,
+			PropModeReplace, (unsigned char*) &bPixel, 1);
+        XClearWindow(app->display(), desktop->handle());
+    }
 }
 
 static void initMenus() {
@@ -347,23 +468,24 @@ static void initMenus() {
     logoutMenu = new YMenu();
     PRECONDITION(logoutMenu != 0);
     logoutMenu->setShared(true); /// !!! get rid of this (refcount objects)
-    logoutMenu->addItem("Logout", 0, "", actionLogout)->setChecked(true);
-    logoutMenu->addItem("Cancel logout", 0, "", actionCancelLogout)->setEnabled(false);
+    logoutMenu->addItem(_("_Logout"), -2, "", actionLogout)->setChecked(true);
+    logoutMenu->addItem(_("_Cancel logout"), -2, "", actionCancelLogout)->setEnabled(false);
     logoutMenu->addSeparator();
 #ifndef NO_CONFIGURE_MENUS
     {
-        const char *c = configArg ? "-c" : 0;
-        char **args = (char **)MALLOC(4 * sizeof(char*));
-        args[0] = newstr("icewm"EXEEXT);
-        args[1] = (char *)c; //!!!
+        char ** args = new (char*)[4];
+        args[0] = newstr(ICEWMEXE);
+        args[1] = configArg ? newstr("-c") : 0;
         args[2] = configArg;
         args[3] = 0;
-        DProgram *re_icewm = DProgram::newProgram("Restart icewm", 0, true, "icewm"EXEEXT, args); //!!!
+        DProgram *re_icewm(DProgram::newProgram
+	    (_("Restart _Icewm"), 0, true, 0, ICEWMEXE, args)); //!!!
         if (re_icewm)
             logoutMenu->add(new DObjectMenuItem(re_icewm));
     }
     {
-        DProgram *re_xterm = DProgram::newProgram("Restart xterm", 0, true, "xterm", 0);
+        DProgram *re_xterm
+	    (DProgram::newProgram(_("Restart _Xterm"), 0, true, 0, "xterm", 0));
         if (re_xterm)
             logoutMenu->add(new DObjectMenuItem(re_xterm));
     }
@@ -377,13 +499,13 @@ static void initMenus() {
     assert(layerMenu != 0);
     layerMenu->setShared(true);
 
-    layerMenu->addItem("Menu", 0, 0, layerActionSet[WinLayerMenu]);
-    layerMenu->addItem("Above Dock", 0, 0, layerActionSet[WinLayerAboveDock]);
-    layerMenu->addItem("Dock", 0, 0, layerActionSet[WinLayerDock]);
-    layerMenu->addItem("OnTop", 0, 0, layerActionSet[WinLayerOnTop]);
-    layerMenu->addItem("Normal", 0, 0, layerActionSet[WinLayerNormal]);
-    layerMenu->addItem("Below", 0, 0, layerActionSet[WinLayerBelow]);
-    layerMenu->addItem("Desktop", 1, 0, layerActionSet[WinLayerDesktop]);
+    layerMenu->addItem(_("_Menu"),       -2, 0, layerActionSet[WinLayerMenu]);
+    layerMenu->addItem(_("_Above Dock"), -2, 0, layerActionSet[WinLayerAboveDock]);
+    layerMenu->addItem(_("_Dock"),       -2, 0, layerActionSet[WinLayerDock]);
+    layerMenu->addItem(_("_OnTop"),      -2, 0, layerActionSet[WinLayerOnTop]);
+    layerMenu->addItem(_("_Normal"),     -2, 0, layerActionSet[WinLayerNormal]);
+    layerMenu->addItem(_("_Below"),      -2, 0, layerActionSet[WinLayerBelow]);
+    layerMenu->addItem(_("D_esktop"),    -2, 0, layerActionSet[WinLayerDesktop]);
 
     moveMenu = new YMenu();
     assert(moveMenu != 0);
@@ -394,27 +516,29 @@ static void initMenus() {
         moveMenu->addItem(s, 0, 0, workspaceActionMoveTo[w]);
     }
 
-    windowMenu->addItem("Restore", 0, KEY_NAME(gKeyWinRestore), actionRestore);
-    windowMenu->addItem("Move", 0, KEY_NAME(gKeyWinMove), actionMove);
-    windowMenu->addItem("Size", 0, KEY_NAME(gKeyWinSize), actionSize);
-    windowMenu->addItem("Minimize", 2, KEY_NAME(gKeyWinMinimize), actionMinimize);
-    windowMenu->addItem("Maximize", 2, KEY_NAME(gKeyWinMaximize), actionMaximize);
-    windowMenu->addItem("Hide", 0, KEY_NAME(gKeyWinHide), actionHide);
-    windowMenu->addItem("Rollup", 4, KEY_NAME(gKeyWinRollup), actionRollup);
+    windowMenu->addItem(_("_Restore"),  -2, KEY_NAME(gKeyWinRestore), actionRestore);
+    windowMenu->addItem(_("_Move"),     -2, KEY_NAME(gKeyWinMove), actionMove);
+    windowMenu->addItem(_("_Size"),     -2, KEY_NAME(gKeyWinSize), actionSize);
+    windowMenu->addItem(_("Mi_nimize"), -2, KEY_NAME(gKeyWinMinimize), actionMinimize);
+    windowMenu->addItem(_("Ma_ximize"), -2, KEY_NAME(gKeyWinMaximize), actionMaximize);
+#ifndef CONFIG_PDA		    
+    windowMenu->addItem(_("_Hide"),     -2, KEY_NAME(gKeyWinHide), actionHide);
+#endif    
+    windowMenu->addItem(_("Roll_up"),   -2, KEY_NAME(gKeyWinRollup), actionRollup);
     windowMenu->addSeparator();
-    windowMenu->addItem("Raise", 4, KEY_NAME(gKeyWinRaise), actionRaise);
-    windowMenu->addItem("Lower", 0, KEY_NAME(gKeyWinLower), actionLower);
-    windowMenu->addSubmenu("Layer", 2, layerMenu);
+    windowMenu->addItem(_("R_aise"),    -2, KEY_NAME(gKeyWinRaise), actionRaise);
+    windowMenu->addItem(_("_Lower"),    -2, KEY_NAME(gKeyWinLower), actionLower);
+    windowMenu->addSubmenu(_("La_yer"), -2, layerMenu);
     if (workspaceCount > 1) {
         windowMenu->addSeparator();
-        windowMenu->addSubmenu("Move To", 5, moveMenu);
-        windowMenu->addItem("Occupy All", 7, KEY_NAME(gKeyWinOccupyAll), actionOccupyAllOrCurrent);
+        windowMenu->addSubmenu(_("Move _To"), -2, moveMenu);
+        windowMenu->addItem(_("Occupy _All"), -2, KEY_NAME(gKeyWinOccupyAll), actionOccupyAllOrCurrent);
     }
     windowMenu->addSeparator();
-    windowMenu->addItem("Close", 0, KEY_NAME(gKeyWinClose), actionClose);
+    windowMenu->addItem(_("_Close"),    -2, KEY_NAME(gKeyWinClose), actionClose);
 #ifdef CONFIG_WINLIST
     windowMenu->addSeparator();
-    windowMenu->addItem("Window list", 0, actionWindowList, windowListMenu);
+    windowMenu->addItem(_("_Window list"), -2, actionWindowList, windowListMenu);
 #endif
 
 #ifndef NO_CONFIGURE_MENUS
@@ -468,18 +592,14 @@ int handler(Display *display, XErrorEvent *xev) {
         xev->request_code == X_ChangeWindowAttributes &&
         xev->error_code == BadAccess)
     {
-        fprintf(stderr, "Another window manager already running, exiting...\n");
+        msg(_("Another window manager already running, exiting..."));
         exit(1);
     }
 
     DBG {
-        char msg[80], req[80], number[80];
+        char message[80], req[80], number[80];
 
-        XGetErrorText(display,
-                      xev->error_code,
-                      msg, sizeof(msg));
         sprintf(number, "%d", xev->request_code);
-
         XGetErrorDatabaseText(display,
                               "XRequest",
                               number, "",
@@ -487,7 +607,13 @@ int handler(Display *display, XErrorEvent *xev) {
         if (!req[0])
             sprintf(req, "[request_code=%d]", xev->request_code);
 
-        fprintf(stderr, "icewm: X-error %s(0x%lX): %s\n", req, xev->resourceid, msg);
+        if (XGetErrorText(display,
+                          xev->error_code,
+                          message, sizeof(message)) !=
+			  Success);
+	    *message = '\0';
+
+        warn(_("X error %s(0x%lX): %s"), req, xev->resourceid, message);
     }
     return 0;
 }
@@ -508,7 +634,9 @@ void dumpZorder(const char *oper, YFrameWindow *w, YFrameWindow *a) {
 
 void runRestart(const char *str, char **args) {
     XSync(app->display(), False);
+    ///!!! problem with repeated SIGHUP for restart...
     app->resetSignals();
+
     if (str) {
         if (args) {
             execvp(str, args);
@@ -517,11 +645,11 @@ void runRestart(const char *str, char **args) {
         }
     } else {
         const char *c = configArg ? "-c" : NULL;
-        execlp("icewm"EXEEXT, "icewm"EXEEXT, c, configArg, 0);
+        execlp(ICEWMEXE, ICEWMEXE, c, configArg, 0);
     }
 
     XBell(app->display(), 100);
-    fprintf(stderr, "icewm: Could not restart %s, not on $PATH?\n", str ? str : "icewm"EXEEXT );
+    warn(_("Could not restart %s, not on $PATH?"), str ? str : ICEWMEXE );
 }
 
 void YWMApp::restartClient(const char *str, char **args) {
@@ -540,6 +668,17 @@ void YWMApp::restartClient(const char *str, char **args) {
     manager->manageClients();
 }
 
+void YWMApp::runOnce(const char *resource, const char *str, char **args) {
+    Window win(manager->findWindow(resource));
+
+    if (win) {
+	YFrameWindow * frame(manager->findFrame(win));
+	if (frame) frame->activate();
+	else XMapRaised(app->display(), win);
+    } else
+	runProgram(str, args);
+}
+
 void YWMApp::actionPerformed(YAction *action, unsigned int /*modifiers*/) {
     if (action == actionLogout) {
         if (!confirmLogout)
@@ -549,8 +688,8 @@ void YWMApp::actionPerformed(YAction *action, unsigned int /*modifiers*/) {
             if (fLogoutMsgBox == 0) {
                 YMsgBox *msgbox = new YMsgBox(YMsgBox::mbOK|YMsgBox::mbCancel);
                 fLogoutMsgBox = msgbox;
-                msgbox->setTitle("Confirm Logout");
-                msgbox->setText("Logout will close all active applications.\nProceed?");
+                msgbox->setTitle(_("Confirm Logout"));
+                msgbox->setText(_("Logout will close all active applications.\nProceed?"));
                 msgbox->autoSize();
                 msgbox->setMsgBoxListener(this);
                 msgbox->showFocused();
@@ -633,9 +772,24 @@ void YWMApp::actionPerformed(YAction *action, unsigned int /*modifiers*/) {
     }
 }
 
-YWMApp::YWMApp(int *argc, char ***argv, const char *displayName): YApplication(argc, argv, displayName) {
+YWMApp::YWMApp(int *argc, char ***argv, const char *displayName): 
+    YApplication(argc, argv, displayName) {
     wmapp = this;
 
+#ifndef LITE
+    if (autoDetectGnome) {
+        if (detectGNOME()) {
+#ifdef CONFIG_TASKBAR
+            showTaskBar = false;
+#endif
+            useRootButtons = 0;
+            DesktopBackgroundColor = 0;
+            DesktopBackgroundPixmap = 0;
+            // !!! more to come, probably
+        }
+    }
+#endif
+
     if (keysFile)
         loadMenus(keysFile, 0);
 
@@ -655,12 +809,14 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName): YApplication(a
     initAtoms();
     initActions();
     initPointers();
+    initFontPath();
     initIconSize();
     initPixmaps();
     initMenus();
 
 #ifndef LITE
     statusMoveSize = new MoveSizeStatus(manager);
+    statusWorkspace = new WorkspaceStatus(manager);
 #endif
 #ifndef LITE
     if (quickSwitch)
@@ -669,7 +825,7 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName): YApplication(a
 #ifdef CONFIG_TASKBAR
     if (showTaskBar) {
         taskBar = new TaskBar(manager);
-        if (taskBar) 
+        if (taskBar)
             taskBar->showBar(true);
     } else {
         taskBar = 0;
@@ -709,9 +865,9 @@ YWMApp::~YWMApp() {
     //delete windowList; windowList = 0;
 #ifndef LITE
     delete switchWindow; switchWindow = 0;
-#endif
-#ifndef LITE
+
     delete statusMoveSize; statusMoveSize = 0;
+    delete statusWorkspace; statusWorkspace = 0;
 #endif
 
 #ifndef NO_CONFIGURE_MENUS
@@ -740,20 +896,38 @@ YWMApp::~YWMApp() {
     delete switchbackPixmap;
     delete logoutPixmap;
 
+#ifdef CONFIG_TASKBAR
+    if (!showTaskBar) {
+        delete taskbuttonactivePixmap;
+        delete taskbuttonminimizedPixmap;
+    }
+#endif
+
     //!!!XFreeGC(display(), outlineGC); lazy init in movesize.cc
     //!!!XFreeGC(display(), clipPixmapGC); in ypaint.cc
+
+    XFlush(display());
 }
 
 void YWMApp::handleSignal(int sig) {
-    if (sig == SIGINT || sig == SIGTERM || sig == SIGQUIT) {
+    switch (sig) {
+    case SIGINT:
+    case SIGTERM:
         actionPerformed(actionExit, 0);
-        return ;
-    }
-    if (sig == SIGHUP) {
+        break;
+
+    case SIGQUIT:
+        actionPerformed(actionLogout, 0);
+        break;
+
+    case SIGHUP:
         restartClient(0, 0);
-        return ;
+        break;
+
+    default:
+        YApplication::handleSignal(sig);
+        break;
     }
-    YApplication::handleSignal(sig);
 }
 
 void YWMApp::handleIdle() {
@@ -809,7 +983,16 @@ int main(int argc, char **argv) {
 #endif
     char *overrideTheme = 0;
 #ifdef I18N
-    setlocale(LC_ALL, "");
+    char *loc = setlocale(LC_ALL, "");
+    if (loc == NULL || !strcmp(loc, "C") || !strcmp(loc, "POSIX") ||
+        !XSupportsLocale())
+        multiByte = false;
+    else
+        multiByte = true;
+#endif
+#ifdef ENABLE_NLS
+    bindtextdomain(PACKAGE, LOCDIR);
+    textdomain(PACKAGE);
 #endif
     for (int i = 1; i < argc; i++) {
         if (argv[i][0] == '-') {
@@ -822,14 +1005,15 @@ int main(int argc, char **argv) {
 #endif
 #ifndef NO_CONFIGURE
             if (strcmp(argv[i], "-c") == 0) {
-                configArg = argv[i];
                 configFile = newstr(argv[++i]);
+                configArg = newstr(configFile);
             } else if (strcmp(argv[i], "-t") == 0)
                 overrideTheme = argv[++i];
             else if (strcmp(argv[i], "-n") == 0)
                 configurationLoaded = 1;
             else if (strcmp(argv[i], "-v") == 0) {
-                fprintf(stderr, "icewm " VERSION ", Copyright 1997-1999 Marko Macek\n");
+                fputs("icewm " VERSION ", Copyright 1997-2001 Marko Macek\n",
+		      stderr);
                 configurationLoaded = 1;
                 exit(0);
             }
@@ -847,25 +1031,13 @@ int main(int argc, char **argv) {
         if (overrideTheme)
             themeName = newstr(overrideTheme);
 
-        if (themeName != 0) {
+        if (themeName != 0) {            
             char *theme = strJoin("themes/", themeName, NULL);
             char *themePath = app->findConfigFile(theme);
             if (themePath)
                 loadConfiguration(themePath);
-            delete theme; theme = 0;
-            delete themePath; themePath = 0;
-        }
-    }
-#endif
-
-#ifndef LITE
-    if (autoDetectGnome) {
-        if (getenv("SESSION_MANAGER") != NULL) { // !!! for now, fix!
-            showTaskBar = false;
-            useRootButtons = 0;
-            DesktopBackgroundColor = 0;
-            DesktopBackgroundPixmap = 0;
-            // !!! more to come, probably
+            delete[] themePath;
+            delete[] theme;
         }
     }
 #endif
