@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 
@@ -18,9 +18,7 @@ static YColor *titleButtonFg = 0;
 
 //!!! get rid of this
 extern YColor *activeTitleBarBg;
-extern YColor *activeTitleBarFg;
 extern YColor *inactiveTitleBarBg;
-extern YColor *inactiveTitleBarFg;
 
 #ifdef CONFIG_LOOK_PIXMAP
 YPixmap *menuButton[2] = { 0, 0 };
@@ -156,8 +154,10 @@ YPixmap *YFrameButton::getImage(int pn) {
         pixmap = restorePixmap[pn];
     else if (fAction == actionClose)
         pixmap = closePixmap[pn];
+#ifndef	CONFIG_PDA
     else if (fAction == actionHide)
         pixmap = hidePixmap[pn];
+#endif	
     else if (fAction == actionRollup)
         pixmap = getFrame()->isRollup() ? rolldownPixmap[pn] : rollupPixmap[pn];
     else if (fAction == actionDepth)
