@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 #include "yfull.h"
@@ -49,7 +49,7 @@ YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win): YW
 
 #ifdef SHAPE
     if (shapesSupported) {
-        XShapeSelectInput (app->display(), handle(), ShapeNotifyMask);
+        XShapeSelectInput(app->display(), handle(), ShapeNotifyMask);
         queryShape();
     }
 #endif
@@ -86,15 +86,15 @@ void YFrameClient::getProtocols() {
 
     fProtocols = fProtocols & wpDeleteWindow; // always keep WM_DELETE_WINDOW
 
-    if (XGetWMProtocols (app->display(),
-                         handle(),
-                         &wmp, &count) && wmp)
+    if (XGetWMProtocols(app->display(),
+                        handle(),
+                        &wmp, &count) && wmp)
     {
         for (int i = 0; i < count; i++) {
             if (wmp[i] == _XA_WM_DELETE_WINDOW) fProtocols |= wpDeleteWindow;
             if (wmp[i] == _XA_WM_TAKE_FOCUS) fProtocols |= wpTakeFocus;
         }
-        XFree (wmp);
+        XFree(wmp);
     }
 }
 
@@ -188,72 +188,72 @@ void YFrameClient::getTransient() {
 
 void YFrameClient::constrainSize(int &w, int &h, long layer, int flags) {
     if (fSizeHints) {
-        int wm = fSizeHints->min_width;
-        int hm = fSizeHints->min_height;
-        int wM = fSizeHints->max_width;
-        int hM = fSizeHints->max_height;
-        int wb = fSizeHints->base_width;
-        int hb = fSizeHints->base_height;
-        int wf = fSizeHints->width_inc;
-        int hf = fSizeHints->height_inc;
+        int const wMin(fSizeHints->min_width);
+        int const hMin(fSizeHints->min_height);
+        int const wMax(fSizeHints->max_width);
+        int const hMax(fSizeHints->max_height);
+        int const wBase(fSizeHints->base_width);
+        int const hBase(fSizeHints->base_height);
+        int const wInc(fSizeHints->width_inc);
+        int const hInc(fSizeHints->height_inc);
 
         if (fSizeHints->flags & PAspect) { // aspect ratios
-            int xm = fSizeHints->min_aspect.x;
-            int ym = fSizeHints->min_aspect.y;
-            int xM = fSizeHints->max_aspect.x;
-            int yM = fSizeHints->max_aspect.y;
+            int const xMin(fSizeHints->min_aspect.x);
+            int const yMin(fSizeHints->min_aspect.y);
+            int const xMax(fSizeHints->max_aspect.x);
+            int const yMax(fSizeHints->max_aspect.y);
 
             // !!! fix handling of KeepX and KeepY together
-            if (xm * h > ym * w) { // min aspect
+            if (xMin * h > yMin * w) { // min aspect
                 if (flags & csKeepX) {
-                    if (h < hm) h = hm;
-                    if (h > hM) h = hM;
-                    h = w * ym / xm;
-                    if (h < hm) h = hm;
-                    if (h > hM) h = hM;
-                    w = h * xm / ym;
+		    w = clamp(w, wMin, wMax);
+                    h = w * yMin / xMin;
+		    h = clamp(h, hMin, hMax);
+                    w = h * xMin / yMin;
                 } else {
-                    if (w > wM) w = wM; // maximum size
-                    if (w < wm) w = wm; // minimum size
-                    w = h * xm / ym;
-                    if (w < wm) w = wm; // minimum size
-                    if (w > wM) w = wM;
-                    h = w * ym / xm;
+		    h = clamp(h, hMin, hMax);
+                    w = h * xMin / yMin;
+		    w = clamp(w, wMin, wMax);
+                    h = w * yMin / xMin;
                 }
             }
-            if (xM * h < yM * w) { // max aspect
+            if (xMax * h < yMax * w) { // max aspect
                 if (flags & csKeepX) {
-                    if (h < hm) h = hm;
-                    if (h > hM) h = hM;
-                    h = w * yM / xM;
-                    if (h > hM) h = hM;
-                    if (h < hm) h = hm;
-                    w = h * xM / yM;
+		    w = clamp(w, wMin, wMax);
+                    h = w * yMax / xMax;
+		    h = clamp(h, hMin, hMax);
+                    w = h * xMax / yMax;
                 } else {
-                    if (w > wM) w = wM; // maximum size
-                    if (w < wm) w = wm; // minimum size
-                    w = h * xM / yM;
-                    if (w < wm) w = wm; // minimum size
-                    if (w > wM) w = wM;
-                    h = w * yM / xM;
+		    h = clamp(h, hMin, hMax);
+                    w = h * xMax / yMax;
+		    w = clamp(w, wMin, wMax);
+                    h = w * yMax / xMax;
                 }
             }
         }
 
-        if (w < wm) w = wm; // minimum size
-        if (h < hm) h = hm;
-
-        if (w > wM) w = wM; // maximum size
-        if (h > hM) h = hM;
+	h = clamp(h, hMin, hMax);
+	w = clamp(w, wMin, wMax);
 
         if (limitSize) {
-            if (w >= int(manager->maxWidth(layer))) w = manager->maxWidth(layer);
-            if (h >= int(manager->maxHeight(layer))) h = manager->maxHeight(layer);
+            w = min(w, manager->maxWidth(layer));
+            h = min(h, manager->maxHeight(layer));
         }
 
-        w = wb + (w - wb + ((flags & csRound) ? wf / 2 : 0)) / wf * wf;
-        h = hb + (h - hb + ((flags & csRound) ? hf / 2 : 0)) / hf * hf;
+#if 0
+        w = wBase + (w - wBase + ((flags & csRound) ? wInc / 2 : 0)) / wInc
+								     * wInc;
+        h = hBase + (h - hBase + ((flags & csRound) ? hInc / 2 : 0)) / hInc
+								     * hInc;
+#else
+	if (flags & csRound) { w+= wInc / 2; h+= hInc / 2; }
+
+	w-= max(0, w - wBase) % wInc;
+	h-= max(0, h - hBase) % hInc;
+#endif								     
+
     }
+
     if (w <= 0) w = 1;
     if (h <= 0) h = 1;
 }
@@ -263,7 +263,7 @@ struct _gravity_offset
   int x, y;
 };
 
-void YFrameClient::gravityOffsets (int &xp, int &yp) {
+void YFrameClient::gravityOffsets(int &xp, int &yp) {
     xp = 0;
     yp = 0;
 
@@ -304,7 +304,7 @@ void YFrameClient::sendMessage(Atom msg, Time timeStamp) {
     xev.format = 32;
     xev.data.l[0] = msg;
     xev.data.l[1] = timeStamp;
-    XSendEvent (app->display(), handle(), False, 0L, (XEvent *) &xev);
+    XSendEvent(app->display(), handle(), False, 0L, (XEvent *) &xev);
 }
 
 void YFrameClient::setFrame(YFrameWindow *newFrame) {
@@ -324,7 +324,7 @@ void YFrameClient::setFrameState(FrameState state) {
     arg[0] = (unsigned long) state;
     arg[1] = (unsigned long) None;
 
-    //printf("setting frame state to %d\n", arg[0]);
+    //msg("setting frame state to %d", arg[0]);
 
     if (state == WithdrawnState) {
         if (phase != phaseRestart && phase != phaseShutdown) {
@@ -402,8 +402,10 @@ void YFrameClient::handleProperty(const XPropertyEvent &property) {
         break;
     case XA_WM_NORMAL_HINTS:
         getSizeHints();
-        if (getFrame())
+        if (getFrame()) {
             getFrame()->updateMwmHints();
+            getFrame()->updateNormalSize();
+	}
         break;
     case XA_WM_TRANSIENT_FOR:
         getTransient();
@@ -488,7 +490,7 @@ void YFrameClient::setWindowTitle(XTextProperty  *prop) {
         setWindowTitle((const char *)prop->value);
         return;
     }
-    status = XmbTextPropertyToTextList (app->display(), prop, &cl, &n);
+    status = XmbTextPropertyToTextList(app->display(), prop, &cl, &n);
     if (status >= Success && n > 0 && cl[0]) {
         setWindowTitle((const char *)cl[0]);
         XFreeStringList(cl);
@@ -514,7 +516,7 @@ void YFrameClient::setIconTitle(XTextProperty  *prop) {
         setIconTitle((const char *)prop->value);
         return;
     }
-    status = XmbTextPropertyToTextList (app->display(), prop, &cl, &n);
+    status = XmbTextPropertyToTextList(app->display(), prop, &cl, &n);
     if (status >= Success && n > 0 && cl[0]) {
         setIconTitle((const char *)cl[0]);
         XFreeStringList(cl);
@@ -692,7 +694,7 @@ long YFrameClient::mwmDecors() {
     long func = mwmFunctions();
 
     if (fMwmHints && (fMwmHints->flags & MWM_HINTS_DECORATIONS)) {
-        if (fMwmHints->decorations & MWM_FUNC_ALL)
+        if (fMwmHints->decorations & MWM_DECOR_ALL)
             decors = ~fMwmHints->decorations;
         else
             decors = fMwmHints->decorations;
@@ -990,7 +992,7 @@ char *YFrameClient::getClientId(Window leader) { /// !!! fix
                            &count, &bytes_remain, (unsigned char **)&cid) == Success && cid)
     {
         if (r_type == XA_STRING && r_format == 8) {
-            //printf("cid=%s\n", cid);
+            //msg("cid=%s", cid);
         } else {
             XFree(cid);
             cid = 0;
