@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998,1999 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  *
  * Dialogs
  */
@@ -17,6 +17,8 @@
 #include "wmapp.h"
 #include "wmmgr.h"
 
+#include "intl.h"
+
 #define HORZ 10
 #define MIDH 10
 #define VERT 10
@@ -50,45 +52,46 @@ CtrlAltDelete::CtrlAltDelete(YWindow *parent): YWindow(parent) {
         cadBg = new YColor(clrDialog);
 
     setStyle(wsOverrideRedirect);
+    setPointer(YApplication::leftPointer);
     setToplevel(true);
  
     b = lockButton = new YActionButton(this);
-    b->setText("Lock Workstation", 5);
+    b->setText(_("Lock _Workstation"), -2);
     if (b->width() > w) w = b->width();
     if (b->height() > h) h = b->height();
     b->setActionListener(this);
     b->show();
 
     b = logoutButton = new YActionButton(this);
-    b->setText("Logout...", 0);
+    b->setText(_("_Logout..."), -2);
     if (b->width() > w) w = b->width();
     if (b->height() > h) h = b->height();
     b->setActionListener(this);
     b->show();
 
     b = cancelButton = new YActionButton(this);
-    b->setText("Cancel", 0);
+    b->setText(_("_Cancel"), -2);
     if (b->width() > w) w = b->width();
     if (b->height() > h) h = b->height();
     b->setActionListener(this);
     b->show();
 
     b = restartButton = new YActionButton(this);
-    b->setText("Restart icewm", 0);
+    b->setText(_("_Restart icewm"), -2);
     if (b->width() > w) w = b->width();
     if (b->height() > h) h = b->height();
     b->setActionListener(this);
     b->show();
 
     b = rebootButton = new YActionButton(this);
-    b->setText("Reboot", 2);
+    b->setText(_("Re_boot"), -2);
     if (b->width() > w) w = b->width();
     if (b->height() > h) h = b->height();
     b->setActionListener(this);
     b->show();
 
     b = shutdownButton = new YActionButton(this);
-    b->setText("Shutdown", 4);
+    b->setText(_("Shut_down"), -2);
     if (b->width() > w) w = b->width();
     if (b->height() > h) h = b->height();
     b->setActionListener(this);
