@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 
 #include "config.h"
@@ -22,6 +22,8 @@
 #include "wmapp.h"
 #include "sysdep.h"
 
+#include "intl.h"
+
 static YColor *activeBorderBg = 0;
 static YColor *inactiveBorderBg = 0;
 
@@ -77,6 +79,7 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
     fKillMsgBox = 0;
 
     setStyle(wsOverrideRedirect);
+    setPointer(YApplication::leftPointer);
 
     PRECONDITION(client != 0);
     fClient = client;
@@ -99,15 +102,20 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
         fMaximizeButton = new YFrameButton(fTitleBar, this, actionMaximize, actionMaximizeVert);
         //fMaximizeButton->setWinGravity(NorthEastGravity);
         fMaximizeButton->show();
-        fMaximizeButton->setToolTip("Maximize");
+        fMaximizeButton->setToolTip(_("Maximize"));
     }
 
     if (!isButton('i'))
         fMinimizeButton = 0;
     else {
-        fMinimizeButton = new YFrameButton(fTitleBar, this, actionMinimize, actionHide);
+        fMinimizeButton = new YFrameButton(fTitleBar, this,
+#ifndef CONFIG_PDA		    
+					   actionMinimize, actionHide);
+#else
+					   actionMinimize, 0);
+#endif
         //fMinimizeButton->setWinGravity(NorthEastGravity);
-        fMinimizeButton->setToolTip("Minimize");
+        fMinimizeButton->setToolTip(_("Minimize"));
         fMinimizeButton->show();
     }
 
@@ -116,26 +124,29 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
     else {
         fCloseButton = new YFrameButton(fTitleBar, this, actionClose, actionKill);
         //fCloseButton->setWinGravity(NorthEastGravity);
-        fCloseButton->setToolTip("Close");
-        if (useXButton)
-            fCloseButton->show();
+        fCloseButton->setToolTip(_("Close"));
+        fCloseButton->show();
     }
 
+#ifndef CONFIG_PDA		    
     if (!isButton('h'))
+#endif
         fHideButton = 0;
+#ifndef CONFIG_PDA		    
     else {
         fHideButton = new YFrameButton(fTitleBar, this, actionHide, actionHide);
         //fHideButton->setWinGravity(NorthEastGravity);
-        fHideButton->setToolTip("Hide");
+        fHideButton->setToolTip(_("Hide"));
         fHideButton->show();
     }
+#endif
 
     if (!isButton('r'))
         fRollupButton = 0;
     else {
         fRollupButton = new YFrameButton(fTitleBar, this, actionRollup, actionRollup);
         //fRollupButton->setWinGravity(NorthEastGravity);
-        fRollupButton->setToolTip("Rollup");
+        fRollupButton->setToolTip(_("Rollup"));
         fRollupButton->show();
     }
 
@@ -144,7 +155,7 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
     else {
         fDepthButton = new YFrameButton(fTitleBar, this, actionDepth, actionDepth);
         //fDepthButton->setWinGravity(NorthEastGravity);
-        fDepthButton->setToolTip("Raise/Lower");
+        fDepthButton->setToolTip(_("Raise/Lower"));
         fDepthButton->show();
     }
 
@@ -184,8 +195,8 @@ YFrameWindow::YFrameWindow(YWindow *parent, YFrameClient *client): YWindow(paren
         fWinListItem = windowList->addWindowListApp(this);
 #endif
     manager->restackWindows(this);
-    if (getLayer() == WinLayerDock)
-        manager->updateWorkArea();
+    if (doNotCover())
+	manager->updateWorkArea();
 #ifdef CONFIG_GUIEVENTS
     wmapp->signalGuiEvent(geWindowOpened);
 #endif
@@ -247,7 +258,7 @@ YFrameWindow::~YFrameWindow() {
             XRemoveFromSaveSet(app->display(), client()->handle());
         XDeleteContext(app->display(), client()->handle(), frameContext);
     }
-    if (getLayer() == WinLayerDock)
+    if (doNotCover())
         manager->updateWorkArea();
 
     delete fClient; fClient = 0;
@@ -281,42 +292,42 @@ void YFrameWindow::createPointerWindows() {
 
     attributes.event_mask = 0;
 
-    attributes.cursor = sizeTopPointer;
+    attributes.cursor = YWMApp::sizeTopPointer.handle();
     topSide = XCreateWindow(app->display(), handle(), 0, 0, 1, 1, 0,
                             CopyFromParent, klass, CopyFromParent,
                             CWCursor | CWEventMask, &attributes);
 
-    attributes.cursor = sizeLeftPointer;
+    attributes.cursor = YWMApp::sizeLeftPointer.handle();
     leftSide = XCreateWindow(app->display(), handle(), 0, 0, 1, 1, 0,
                             CopyFromParent, klass, CopyFromParent,
                             CWCursor | CWEventMask, &attributes);
 
-    attributes.cursor = sizeRightPointer;
+    attributes.cursor = YWMApp::sizeRightPointer.handle();
     rightSide = XCreateWindow(app->display(), handle(), 0, 0, 1, 1, 0,
                             CopyFromParent, klass, CopyFromParent,
                             CWCursor | CWEventMask, &attributes);
 
-    attributes.cursor = sizeBottomPointer;
+    attributes.cursor = YWMApp::sizeBottomPointer.handle();
     bottomSide = XCreateWindow(app->display(), handle(), 0, 0, 1, 1, 0,
                             CopyFromParent, klass, CopyFromParent,
                             CWCursor | CWEventMask, &attributes);
 
-    attributes.cursor = sizeTopLeftPointer;
+    attributes.cursor = YWMApp::sizeTopLeftPointer.handle();
     topLeftCorner = XCreateWindow(app->display(), handle(), 0, 0, 1, 1, 0,
                                   CopyFromParent, klass, CopyFromParent,
                                   CWCursor | CWEventMask, &attributes);
 
-    attributes.cursor = sizeTopRightPointer;
+    attributes.cursor = YWMApp::sizeTopRightPointer.handle();
     topRightCorner = XCreateWindow(app->display(), handle(), 0, 0, 1, 1, 0,
                                    CopyFromParent, klass, CopyFromParent,
                                    CWCursor | CWEventMask, &attributes);
 
-    attributes.cursor = sizeBottomLeftPointer;
+    attributes.cursor = YWMApp::sizeBottomLeftPointer.handle();
     bottomLeftCorner = XCreateWindow(app->display(), handle(), 0, 0, 1, 1, 0,
                                      CopyFromParent, klass, CopyFromParent,
                                      CWCursor | CWEventMask, &attributes);
 
-    attributes.cursor = sizeBottomRightPointer;
+    attributes.cursor = YWMApp::sizeBottomRightPointer.handle();
     bottomRightCorner = XCreateWindow(app->display(), handle(), 0, 0, 1, 1, 0,
                                       CopyFromParent, klass, CopyFromParent,
                                       CWCursor | CWEventMask, &attributes);
@@ -437,10 +448,10 @@ void YFrameWindow::configureClient(const XConfigureRequestEvent &configureReques
             default:
                 return ;
             }
-            XConfigureWindow (app->display(),
-                              handle(),
-                              configureRequest.value_mask & (CWSibling | CWStackMode),
-                              &xwc);
+            XConfigureWindow(app->display(),
+                             handle(),
+                             configureRequest.value_mask & (CWSibling | CWStackMode),
+                             &xwc);
         } else if (xwc.sibling == None && manager->top(getLayer()) != 0) {
             switch (xwc.stack_mode) {
             case Above:
@@ -452,6 +463,29 @@ void YFrameWindow::configureClient(const XConfigureRequestEvent &configureReques
                    (clickFocus || !strongPointerFocus))
                     activate();
 #endif
+                { /* warning, tcl/tk "fix" here */
+                    XEvent xev;
+
+                    memset(&xev, 0, sizeof(xev));
+                    xev.xconfigure.type = ConfigureNotify;
+                    xev.xconfigure.display = app->display();
+                    xev.xconfigure.event = handle();
+                    xev.xconfigure.window = handle();
+                    xev.xconfigure.x = x();
+                    xev.xconfigure.y = y();
+                    xev.xconfigure.width = width();
+                    xev.xconfigure.height = height();
+                    xev.xconfigure.border_width = 0;
+
+                    xev.xconfigure.above = None;
+                    xev.xconfigure.override_redirect = False;
+
+                    XSendEvent(app->display(),
+                               handle(),
+                               False,
+                               StructureNotifyMask,
+                               &xev);
+                }
                 break;
             case Below:
                 wmLower();
@@ -478,7 +512,7 @@ void YFrameWindow::configureClient(int cx, int cy, int cwidth, int cheight) {
             fWinState &= ~(WinStateMaximizedVert | WinStateMaximizedHoriz);
             if (fMaximizeButton) {
                 fMaximizeButton->setActions(actionMaximize, actionMaximizeVert);
-                fMaximizeButton->setToolTip("Maximize");
+                fMaximizeButton->setToolTip(_("Maximize"));
             }
         }
 #endif
@@ -498,6 +532,7 @@ void YFrameWindow::configureClient(int cx, int cy, int cwidth, int cheight) {
         int nw = cwidth;
         int nh = cheight;
         XSizeHints *sh = client()->sizeHints();
+
         bool cxw = true;
         bool cy = true;
         bool ch = true;
@@ -853,9 +888,11 @@ void YFrameWindow::actionPerformed(YAction *action, unsigned int modifiers) {
             wmClose();
     } else if (action == actionKill) {
         wmConfirmKill();
+#ifndef CONFIG_PDA		    
     } else if (action == actionHide) {
         if (canHide())
             wmHide();
+#endif	    
     } else if (action == actionShow) {
         wmShow();
     } else if (action == actionMove) {
@@ -945,6 +982,44 @@ void YFrameWindow::wmMinimize() {
         manager->focusTopWindow();
 }
 
+void YFrameWindow::minimizeTransients() {
+    for (YFrameWindow *w = transient(); w; w = w->nextTransient()) {
+// Since a) YFrameWindow::setState is too heavy but b) we want to save memory
+	MSG(("> isMinimized: %d\n", w->isMinimized()));
+        if (w->isMinimized())
+            w->fWinState|= WinStateWasMinimized;
+        else
+            w->fWinState&= ~WinStateWasMinimized;
+	MSG(("> wasMinimized: %d\n", w->wasMinimized()));
+        if (!w->isMinimized()) w->wmMinimize();
+    }
+}
+
+void YFrameWindow::restoreMinimizedTransients() {
+    for (YFrameWindow *w = transient(); w; w = w->nextTransient())
+        if (w->isMinimized() && !w->wasMinimized())
+            w->setState(WinStateMinimized, 0);
+}
+
+void YFrameWindow::hideTransients() {
+    for (YFrameWindow *w = transient(); w; w = w->nextTransient()) {
+// See YFrameWindow::minimizeTransients() for reason
+	MSG(("> isHidden: %d\n", w->isHidden()));
+        if (w->isHidden())
+            w->fWinState|= WinStateWasHidden;
+        else
+            w->fWinState&= ~WinStateWasHidden;
+	MSG(("> was visible: %d\n", w->wasHidden());
+        if (!w->isHidden()) w->wmHide());
+    }
+}
+
+void YFrameWindow::restoreHiddenTransients() {
+    for (YFrameWindow *w = transient(); w; w = w->nextTransient())
+        if (w->isHidden() && !w->wasHidden())
+            w->setState(WinStateHidden, 0);
+}
+
 void YFrameWindow::DoMaximize(long flags) {
     setState(WinStateRollup, 0);
 
@@ -1072,13 +1147,10 @@ void YFrameWindow::doRaise() {
 #endif
     if (this != manager->top(getLayer())) {
         setAbove(manager->top(getLayer()));
-        {
-            YFrameWindow *w = transient();
-            while (w) {
-                w->doRaise();
-                w = w->nextTransient();
-            }
-        }
+
+	for (YFrameWindow * w (transient()); w; w = w->nextTransient())
+	    w->doRaise();
+
 #ifdef DEBUG
         if (debug_z) dumpZorder("wmRaise after raise: ", this);
 #endif
@@ -1089,27 +1161,31 @@ void YFrameWindow::wmClose() {
     if (!canClose())
         return ;
 
+#ifdef CONFIG_PDA
+    wmHide();
+#else
     XGrabServer(app->display());
     client()->getProtocols();
 
-    if (client()->protocols() & YFrameClient::wpDeleteWindow)
+    if (client()->protocols() & YFrameClient::wpDeleteWindow) {
         client()->sendMessage(_XA_WM_DELETE_WINDOW);
-    else {
+    } else {
         wmConfirmKill();
     }
     XUngrabServer(app->display());
+#endif    
 }
 
 void YFrameWindow::wmConfirmKill() {
 #ifndef LITE
     if (fKillMsgBox == 0) {
         YMsgBox *msgbox = new YMsgBox(YMsgBox::mbOK|YMsgBox::mbCancel);
-        char *title = strJoin("Kill Client: ", getTitle(), 0);
+        char *title = strJoin(_("Kill Client: "), getTitle(), 0);
         fKillMsgBox = msgbox;
 
         msgbox->setTitle(title);
         delete title; title = 0;
-        msgbox->setText("Warning! Unsaved changes will be lost!\nProceed?");
+        msgbox->setText(_("Warning! Unsaved changes will be lost!\nProceed?"));
         msgbox->autoSize();
         msgbox->setMsgBoxListener(this);
         msgbox->showFocused();
@@ -1235,10 +1311,11 @@ void YFrameWindow::focus(bool canWarp) {
     if (!visibleOn(manager->activeWorkspace()))
         manager->activateWorkspace(getWorkspace());
     // recover lost (offscreen) windows !!!
-    if (x() >= int(manager->width()) ||
-        y() >= int(manager->height()) ||
-        x() <= - int(width()) ||
-        y() <= - int(height()))
+    if (limitPosition &&
+        (x() >= int(manager->width()) ||
+         y() >= int(manager->height()) ||
+         x() <= - int(width()) ||
+         y() <= - int(height())))
     {
         int newX = x();
         int newY = y();
@@ -1448,7 +1525,7 @@ void YFrameWindow::wmOccupyAllOrCurrent() {
 
 void YFrameWindow::wmOccupyAll() {
     setSticky(!isSticky());
-    if (getLayer() == WinLayerDock)
+    if (doNotCover())
         manager->updateWorkArea();
 #ifdef CONFIG_TASKBAR
     if (taskBar && taskBar->taskPane())
@@ -1476,6 +1553,11 @@ void YFrameWindow::getFrameHints() {
     long decors = client()->mwmDecors();
     long functions = client()->mwmFunctions();
     long win_hints = client()->winHints();
+    MwmHints *mwm_hints = client()->mwmHints();
+    int functions_only = (mwm_hints &&
+                      (mwm_hints->flags & (MWM_HINTS_FUNCTIONS |
+                                           MWM_HINTS_FUNCTIONS))
+                      == MWM_HINTS_FUNCTIONS);
 
     fFrameFunctions = 0;
     fFrameDecors = 0;
@@ -1488,14 +1570,29 @@ void YFrameWindow::getFrameHints() {
     if (decors & MWM_DECOR_MAXIMIZE)    fFrameDecors |= fdMaximize;
     if (decors & MWM_DECOR_MINIMIZE)    fFrameDecors |= fdMinimize | fdHide | fdRollup;
 
-    if (functions & MWM_FUNC_MOVE)      fFrameFunctions |= ffMove;
-    if (functions & MWM_FUNC_RESIZE)    fFrameFunctions |= ffResize;
-    if (functions & MWM_FUNC_MAXIMIZE)  fFrameFunctions |= ffMaximize;
-    if (functions & MWM_FUNC_MINIMIZE)
+    if (functions & MWM_FUNC_MOVE) {
+        fFrameFunctions |= ffMove;
+        if (functions_only)
+            fFrameDecors |= fdBorder;
+    }
+    if (functions & MWM_FUNC_RESIZE)    {
+        fFrameFunctions |= ffResize;
+        if (functions_only)
+            fFrameDecors |= fdResize | fdBorder;
+    }
+    if (functions & MWM_FUNC_MAXIMIZE) {
+        fFrameFunctions |= ffMaximize;
+        if (functions_only)
+            fFrameDecors |= fdMaximize;
+    }
+    if (functions & MWM_FUNC_MINIMIZE) {
         fFrameFunctions |= ffMinimize | ffHide | ffRollup;
+        if (functions_only)
+            fFrameDecors |= fdMinimize | fdHide | fdRollup;
+    }
     if (functions & MWM_FUNC_CLOSE) {
         fFrameFunctions |= ffClose;
-        fFrameDecors |= fdClose; /* hack */
+        fFrameDecors |= fdClose;
     }
 #else
     fFrameFunctions =
@@ -1513,13 +1610,15 @@ void YFrameWindow::getFrameHints() {
         fFrameOptions |= foIgnoreWinList;
     if (win_hints & WinHintsSkipTaskBar)
         fFrameOptions |= foIgnoreTaskBar;
+    if (win_hints & WinHintsDoNotCover)
+        fFrameOptions |= foDoNotCover;
 
 #ifndef NO_WINDOW_OPTIONS
     WindowOption wo;
 
     getWindowOptions(wo, false);
 
-    /*fprintf(stderr, "decor: %lX %lX %lX %lX %lX %lX",
+    /*msg("decor: %lX %lX %lX %lX %lX %lX",
             wo.function_mask, wo.functions,
             wo.decor_mask, wo.decors,
             wo.option_mask, wo.options);*/
@@ -1635,11 +1734,15 @@ YIcon *newClientIcon(int count, int reclen, long *elem) {
         if (depth == (unsigned)DefaultDepth(app->display(),
                                             DefaultScreen(app->display())))
         {
-            //printf("%d %d\n", w, h);
+#ifdef CONFIG_IMLIB
+            small = new YPixmap(pixmap, mask, w, h, ICON_SMALL, ICON_SMALL);
+	    large = new YPixmap(pixmap, mask, w, h, ICON_LARGE, ICON_LARGE);
+#else
             if (w == ICON_SMALL)
                 small = new YPixmap(pixmap, mask, w, h);
             else if (w == ICON_LARGE)
                 large = new YPixmap(pixmap, mask, w, h);
+#endif
         }
     }
     if (small || large)
@@ -1699,22 +1802,20 @@ void YFrameWindow::updateIcon() {
 #endif
 
 YFrameWindow *YFrameWindow::nextLayer() {
-    if (fNextFrame)
-        return fNextFrame;
-    long l = getLayer();
-    while (l-- > 0)
-        if (manager->top(l))
-            return manager->top(l);
+    if (fNextFrame) return fNextFrame;
+	
+    for (long l(getLayer() - 1); l > 0; --l)
+        if (manager->top(l)) return manager->top(l);
+
     return 0;
 }
 
 YFrameWindow *YFrameWindow::prevLayer() {
-    if (fPrevFrame)
-        return fPrevFrame;
-    long l = getLayer();
-    while (++l < WinLayerCount)
-        if (manager->bottom(l))
-            return manager->bottom(l);
+    if (fPrevFrame) return fPrevFrame;
+    
+    for (long l(getLayer() + 1); l < WinLayerCount; ++l)
+        if (manager->bottom(l)) return manager->bottom(l);
+
     return 0;
 }
 
@@ -1726,21 +1827,18 @@ YMenu *YFrameWindow::windowMenu() {
 }
 
 void YFrameWindow::addAsTransient() {
-    Window fTransientFor = client()->ownerWindow();
-    if (fTransientFor) {
-        fOwner = manager->findFrame(fTransientFor);
-        if (fOwner != 0) {
-            MSG(("transient for 0x%lX: 0x%lX", fTransientFor, fOwner));
-            if (fOwner) {
-                fNextTransient = fOwner->transient();
-                fOwner->setTransient(this);
-            } else {
-                fTransientFor = 0; // ?
+    Window groupLeader(client()->ownerWindow());
+    
+    if (groupLeader) {
+        fOwner = manager->findFrame(groupLeader);
 
-                fNextTransient = 0;
-                fOwner = 0;
-            }
-        }
+        if (fOwner) {
+            MSG(("transient for 0x%lX: 0x%lX", groupLeader, fOwner));
+	    PRECONDITION(fOwner->transient() != this);
+
+            fNextTransient = fOwner->transient();
+            fOwner->setTransient(this);
+	}
     }
 }
 
@@ -1748,30 +1846,23 @@ void YFrameWindow::removeAsTransient() {
     if (fOwner) {
         MSG(("removeAsTransient"));
 
-        YFrameWindow *w = fOwner->transient(), *cp = 0;
-
-        while (w) {
-            if (w == this) {
-                if (cp)
-                    cp->setNextTransient(nextTransient());
-                else
-                    fOwner->setTransient(nextTransient());
+        for (YFrameWindow * curr(fOwner->transient()), * prev(NULL);
+	     curr; prev = curr, curr = curr->nextTransient()) {
+	    if (curr == this) {
+                if (prev) prev->setNextTransient(nextTransient());
+                else fOwner->setTransient(nextTransient());
+		break;
             }
-            w = w->nextTransient();
         }
-        fOwner = 0;
-        fNextTransient = 0;
+
+        fOwner = NULL;
+        fNextTransient = NULL;
     }
 }
 
 void YFrameWindow::addTransients() {
-    YFrameWindow *w = manager->bottomLayer();
-
-    while (w) {
-        if (w->owner() == 0)
-            w->addAsTransient();
-        w = w->prevLayer();
-    }
+    for (YFrameWindow * w(manager->bottomLayer()); w; w = w->prevLayer())
+        if (w->owner() == 0) w->addAsTransient();
 }
 
 void YFrameWindow::removeTransients() {
@@ -1865,7 +1956,10 @@ void YFrameWindow::setLayer(long layer) {
         insertFrame();
         client()->setWinLayerHint(fWinLayer);
         manager->restackWindows(this);
-        if (getLayer() == WinLayerDock || oldLayer == WinLayerDock)
+
+        if (limitByDockLayer &&
+	   (getLayer() == WinLayerDock ||
+	      oldLayer == WinLayerDock))
             manager->updateWorkArea();
     }
 }
@@ -1995,51 +2089,61 @@ void YFrameWindow::updateNormalSize() {
             normalHeight = sh ? (nh - sh->base_height) / sh->height_inc : nh;
     }
 
-    //fprintf(stderr, "setNormal: (%d:%d %dx%d) icon (%d:%d)\n", normalX, normalY, normalWidth, normalHeight, iconX, iconY);
+    MSG(("updateNormalSize: (%d:%d %dx%d) icon (%d:%d)", 
+    	 normalX, normalY, normalWidth, normalHeight, iconX, iconY));
 }
 
 void YFrameWindow::updateLayout() {
-    XSizeHints *sh = client()->sizeHints();
-    int nx = normalX;
-    int ny = normalY;
-    int nw = sh ? normalWidth * sh->width_inc + sh->base_width : normalWidth;
-    int nh = sh ? normalHeight * sh->height_inc + sh->base_height : normalHeight;
-
     if (isIconic()) {
         if (iconX == -1 && iconY == -1)
             manager->getIconPosition(this, &iconX, &iconY);
-        nx = iconX;
-        ny = iconY;
-        nw = fMiniIcon->width();
-        nh = fMiniIcon->height();
+
+	setGeometry(iconX, iconY, fMiniIcon->width(), fMiniIcon->height());
     } else {
-        if (isMaximizedHoriz()) {
+	XSizeHints *sh(client()->sizeHints());
+
+	int nx(normalX);
+	int ny(normalY);
+
+	int nw(sh ? normalWidth * sh->width_inc + sh->base_width
+		  : normalWidth);
+	int nh(sh ? normalHeight * sh->height_inc + sh->base_height
+		  : normalHeight);
+
+        if (isMaximizedHoriz())
             nw = manager->maxWidth(getLayer());
 
-            if (nx + nw > int(manager->maxX(getLayer())))
-                nx = manager->minX(getLayer());
-            if (nx < manager->minX(getLayer()))
-                nx = manager->minX(getLayer());
-        }
-        if (isMaximizedVert()) {
+        if (isMaximizedVert())
             nh = manager->maxHeight(getLayer()) - titleY();
 
-            if (ny + nh + int(wsTitleBar) > int(manager->maxY(getLayer())))
-                ny = manager->minY(getLayer());
-            if (ny < manager->minY(getLayer()))
-                ny = manager->minY(getLayer());
-        }
+	if (!doNotCover()) {
+	    nx = min(nx, manager->maxX(getLayer()) - nw);
+	    nx = max(nx, manager->minX(getLayer()));
+	    nw = min(nw, manager->maxX(getLayer()) - nx);
+
+	    ny = min(ny, manager->maxY(getLayer()) - nh - int(titleY()));
+	    ny = max(ny, manager->minY(getLayer()));
+	    nh = min(nh, manager->maxY(getLayer()) - ny - int(titleY()));
+	}
+
         client()->constrainSize(nw, nh, getLayer());
 
         if (isRollup())
             nh = 0;
 
-        nx -= borderX();
-        ny -= borderY();
-        nw += 2 * borderX();
-        nh += 2 * borderY() + titleY();
+	if (!(isMaximizedHoriz() && considerHorizBorder)) {
+	    nx -= borderX();
+	    nw += 2 * borderX();
+	}
+
+	bool const cv(!(isMaximizedVert() && considerVertBorder));
+	bool const ch(cv || isRollup());
+
+	if (cv) ny -= borderY();
+	if (ch) nh += 2 * borderY();
+	
+	setGeometry(nx, ny, nw, nh + titleY());
     }
-    setGeometry(nx, ny, nw, nh);
 }
 
 void YFrameWindow::setState(long mask, long state) {
@@ -2064,7 +2168,7 @@ void YFrameWindow::setState(long mask, long state) {
     }
     MSG(("setState: oldState: %lX, newState: %lX, mask: %lX, state: %lX",
          fOldState, fNewState, mask, state));
-    //fprintf(stderr, "normal1: (%d:%d %dx%d)\n", normalX, normalY, normalWidth, normalHeight);
+    //msg("normal1: (%d:%d %dx%d)", normalX, normalY, normalWidth, normalHeight);
     if ((fOldState ^ fNewState) & (WinStateMaximizedVert |
                                    WinStateMaximizedHoriz))
     {
@@ -2073,14 +2177,18 @@ void YFrameWindow::setState(long mask, long state) {
         if (fMaximizeButton)
             if (isMaximized()) {
                 fMaximizeButton->setActions(actionRestore, actionRestore);
-                fMaximizeButton->setToolTip("Restore");
+                fMaximizeButton->setToolTip(_("Restore"));
             } else {
                 fMaximizeButton->setActions(actionMaximize, actionMaximizeVert);
-                fMaximizeButton->setToolTip("Maximize");
+                fMaximizeButton->setToolTip(_("Maximize"));
             }
     }
     if ((fOldState ^ fNewState) & WinStateMinimized) {
         MSG(("WinStateMinimized: %d", isMaximized()));
+        if (fNewState & WinStateMinimized)
+            minimizeTransients();
+        else if (owner() && owner()->isMinimized())
+            owner()->setState(WinStateMinimized, 0);
 
         if (minimizeToDesktop && fMiniIcon) {
             if (isIconic()) {
@@ -2102,22 +2210,28 @@ void YFrameWindow::setState(long mask, long state) {
         MSG(("WinStateRollup: %d", isRollup()));
         if (fRollupButton) {
             if (isRollup()) {
-                fRollupButton->setToolTip("Rolldown");
+                fRollupButton->setToolTip(_("Rolldown"));
             } else {
-                fRollupButton->setToolTip("Rollup");
+                fRollupButton->setToolTip(_("Rollup"));
             }
             fRollupButton->repaint();
         }
     }
     if ((fOldState ^ fNewState) & WinStateHidden) {
         MSG(("WinStateHidden: %d", isHidden()));
+
+        if (fNewState & WinStateHidden)
+            hideTransients();
+        else if (owner() && owner()->isHidden())
+            owner()->setState(WinStateHidden, 0);
+
 #ifdef CONFIG_TASKBAR
         updateTaskBar();
 #endif
     }
 #if 0 //!!!
     if ((fOldState ^ fNewState) & WinStateDockHorizontal) {
-        if (getLayer() == WinLayerDock)
+        if (doNotCover())
             manager->updateWorkArea();
     }
 #endif
@@ -2129,10 +2243,17 @@ void YFrameWindow::setState(long mask, long state) {
     if ((fOldState ^ fNewState) & (WinStateRollup | WinStateMinimized))
         setShape();
 #endif
+    if ((fOldState ^ fNewState) & WinStateMinimized) {
+        if (!(fNewState & WinStateMinimized))
+            restoreMinimizedTransients();
+    }
+    if ((fOldState ^ fNewState) & WinStateHidden) {
+        if (!(fNewState & WinStateHidden))
+            restoreHiddenTransients();
+    }
     if ((clickFocus || !strongPointerFocus) &&
         this == manager->getFocus() &&
-        ((fOldState ^ fNewState) & WinStateRollup))
-    {
+        ((fOldState ^ fNewState) & WinStateRollup)) {
         manager->setFocus(this);
     }
 }
@@ -2210,7 +2331,7 @@ void YFrameWindow::updateTaskBar() {
 #endif
 
 void YFrameWindow::handleMsgBox(YMsgBox *msgbox, int operation) {
-    //printf("msgbox operation %d\n", operation);
+    //msg("msgbox operation %d", operation);
     if (msgbox == fKillMsgBox && fKillMsgBox) {
         if (fKillMsgBox) {
             manager->unmanageClient(fKillMsgBox->handle());
