@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1988 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 #include "yfull.h"
@@ -40,6 +40,7 @@ YWindowManager::YWindowManager(YWindow *parent, Window win): YDesktop(parent, wi
     }
     fColormapWindow = 0;
     fActiveWorkspace = WinWorkspaceInvalid;
+    fLastWorkspace = WinWorkspaceInvalid;
     fArrangeCount = 0;
     fArrangeInfo = 0;
     fMinX = 0;
@@ -52,7 +53,7 @@ YWindowManager::YWindowManager(YWindow *parent, Window win): YDesktop(parent, wi
     clientContext = XUniqueContext();
 
     setStyle(wsManager);
-    setPointer(leftPointer);
+    setPointer(YApplication::leftPointer);
 
     fTopWin = new YWindow();;
     if (edgeWorkspaceSwitching) {
@@ -92,9 +93,11 @@ void YWindowManager::grabKeys() {
 
     GRAB_WMKEY(gKeySysWorkspacePrev);
     GRAB_WMKEY(gKeySysWorkspaceNext);
+    GRAB_WMKEY(gKeySysWorkspaceLast);
 
     GRAB_WMKEY(gKeySysWorkspacePrevTakeWin);
     GRAB_WMKEY(gKeySysWorkspaceNextTakeWin);
+    GRAB_WMKEY(gKeySysWorkspaceLastTakeWin);
 
     GRAB_WMKEY(gKeySysWinMenu);
     GRAB_WMKEY(gKeySysMenu);
@@ -217,10 +220,15 @@ bool YWindowManager::handleKey(const XKeyEvent &key) {
         } else if (IS_WMKEY(k, vm, gKeySysWorkspaceNext)) {
             XUngrabKeyboard(app->display(), CurrentTime);
             switchToNextWorkspace(false);
+        } else if (IS_WMKEY(k, vm, gKeySysWorkspaceLast)) {
+            XUngrabKeyboard(app->display(), CurrentTime);
+            switchToLastWorkspace(false);
         } else if (IS_WMKEY(k, vm, gKeySysWorkspacePrevTakeWin)) {
             switchToPrevWorkspace(true);
         } else if (IS_WMKEY(k, vm, gKeySysWorkspaceNextTakeWin)) {
             switchToNextWorkspace(true);
+        } else if (IS_WMKEY(k, vm, gKeySysWorkspaceLastTakeWin)) {
+            switchToLastWorkspace(true);
         } else if (IS_WMKEY(k, vm, gKeySysWorkspace1)) {
             switchToWorkspace(0, false);
         } else if (IS_WMKEY(k, vm, gKeySysWorkspace2)) {
@@ -283,7 +291,7 @@ bool YWindowManager::handleKey(const XKeyEvent &key) {
         } else {
             KProgram *p = keyProgs;
             while (p) {
-                //printf("%X=%X %X=%X\n", k, p->key(), vm, p->modifiers());
+                //msg("%X=%X %X=%X", k, p->key(), vm, p->modifiers());
                 if (p->isKey(k, vm))
                     p->open();
                 p = p->getNext();
@@ -451,6 +459,60 @@ void YWindowManager::handleClientMessage(const XClientMessageEvent &message) {
     }
 }
 
+Window YWindowManager::findWindow(char const * resource) {
+    char *wmInstance(0), *wmClass(0);
+    
+    char const * dot(resource ? strchr(resource, '.') : 0);
+
+    if (dot) {
+	wmInstance = (dot != resource ? newstr(resource, dot - resource) : 0);
+	wmClass = newstr(dot + 1);
+    } else if (resource)
+	wmInstance = newstr(resource);
+
+    Window win = findWindow(desktop->handle(), wmInstance, wmClass);
+
+    delete[] wmClass;
+    delete[] wmInstance;
+    
+    return win;
+}
+
+Window YWindowManager::findWindow(Window root, char const * wmInstance,
+				  char const * wmClass) {
+    Window firstMatch = None;
+    Window parent, *clients;
+    unsigned nClients;
+
+    XQueryTree(app->display(), root, &root, &parent, &clients, &nClients);
+
+    if (clients) {
+	unsigned n;
+	
+	for (n = 0; !firstMatch && n < nClients; ++n) {
+	    XClassHint wmclass;
+
+	    if (XGetClassHint(app->display(), clients[n], &wmclass)) {
+		if ((wmInstance == NULL ||
+		    strcmp(wmInstance, wmclass.res_name) == 0) &&
+		    (wmClass == NULL || 
+		    strcmp(wmClass, wmclass.res_class) == 0))
+		    firstMatch = clients[n];
+		    
+		XFree(wmclass.res_name);
+		XFree(wmclass.res_class);
+	    }
+	    
+	    if (!firstMatch)
+		firstMatch = findWindow(clients[n], wmInstance, wmClass);
+	}
+
+	XFree(clients);
+    }
+    
+    return firstMatch;
+}
+
 YFrameWindow *YWindowManager::findFrame(Window win) {
     YFrameWindow *frame;
 
@@ -471,38 +533,42 @@ YFrameClient *YWindowManager::findClient(Window win) {
         return 0;
 }
 
+#ifndef LITE
 void YWindowManager::setFocus(YFrameWindow *f, bool canWarp) {
+#else
+void YWindowManager::setFocus(YFrameWindow *f, bool /*canWarp*/) {
+#endif
     YFrameClient *c = f ? f->client() : 0;
     Window w = desktop->handle();
 
-    //fprintf(stderr, "SET FOCUS f=%lX\n", f);
+    MSG(("SET FOCUS f=%lX", f));
 
     if (f == 0) {
         YFrameWindow *ff = getFocus();
-        if (ff)
-            switchFocusFrom(ff);
+        if (ff) switchFocusFrom(ff);
     }
 
     if (f && f->visible()) {
-        if (c && c->visible() && !f->isRollup() && !f->isIconic())
+        if (c && c->visible() && !(f->isRollup() || f->isIconic()))
             w = c->handle();
         else
             w = f->handle();
+
         switchFocusTo(f);
     }
 #if 0
     if (w == desktop->handle()) {
-        fprintf(stderr, "%lX Focus 0x%lX desktop\n",
-                app->getEventTime(), w);
+        msg("%lX Focus 0x%lX desktop",
+            app->getEventTime(), w);
     } else if (f && w == f->handle()) {
-        fprintf(stderr, "%lX Focus 0x%lX frame %s\n",
-                app->getEventTime(), w, f->getTitle());
+        msg("%lX Focus 0x%lX frame %s",
+            app->getEventTime(), w, f->getTitle());
     } else if (f && c && w == c->handle()) {
-        fprintf(stderr, "%lX Focus 0x%lX client %s\n",
-                app->getEventTime(), w, f->getTitle());
+        msg("%lX Focus 0x%lX client %s",
+            app->getEventTime(), w, f->getTitle());
     } else {
-        fprintf(stderr, "%lX Focus 0x%lX\n",
-                app->getEventTime(), w);
+        msg("%lX Focus 0x%lX",
+            app->getEventTime(), w);
     }
 #endif
     XSetInputFocus(app->display(), w, RevertToNone, CurrentTime);
@@ -515,17 +581,12 @@ void YWindowManager::setFocus(YFrameWindow *f, bool canWarp) {
 
 #ifndef LITE
     /// !!! /* warp pointer sucks */
-    if (canWarp &&
-        !clickFocus &&
-        warpPointer &&
-        phase == phaseRunning)
-    {
-        XWarpPointer(app->display(), None, handle(),
-                     0, 0, 0, 0,
+    if (f && canWarp && !clickFocus && warpPointer && phase == phaseRunning)
+        XWarpPointer(app->display(), None, handle(), 0, 0, 0, 0,
                      f->x() + f->borderX(), f->y() + f->borderY() + f->titleY());
-    }
+
 #endif
-    //fprintf(stderr, "SET FOCUS END\n");
+    MSG(("SET FOCUS END"));
 }
 
 void YWindowManager::loseFocus(YFrameWindow *window) {
@@ -540,7 +601,7 @@ void YWindowManager::loseFocus(YFrameWindow *window) {
                                          YFrameWindow::fwfLayers |
                                          YFrameWindow::fwfWorkspace);
 
-    PRECONDITION (w != window);
+    PRECONDITION(w != window);
     setFocus(w, false);
 }
 
@@ -571,7 +632,7 @@ void YWindowManager::loseFocus(YFrameWindow *window,
 
     if (w == window)
         w = 0;
-    //printf("loseFocus to %s\n", w ? w->getTitle() : "<none>");
+    //msg("loseFocus to %s", w ? w->getTitle() : "<none>");
     setFocus(w, false);
 }
 
@@ -723,7 +784,7 @@ int YWindowManager::calcCoverage(bool down, YFrameWindow *frame, int x, int y, i
         if (factor > 1)
             factor /= 2;
     }
-    //printf("coverage %d %d %d %d = %d\n", x, y, w, h, cover);
+    //msg("coverage %d %d %d %d = %d", x, y, w, h, cover);
     return cover;
 }
 
@@ -744,7 +805,7 @@ void YWindowManager::tryCover(bool down, YFrameWindow *frame, int x, int y, int
 
     ncover = calcCoverage(down, frame, x, y, w, h);
     if (ncover < cover) {
-        //printf("min: %d %d %d\n", ncover, x, y);
+        //msg("min: %d %d %d", ncover, x, y);
         px = x;
         py = y;
         cover = ncover;
@@ -905,7 +966,7 @@ void YWindowManager::getNewPosition(YFrameWindow *frame, int &x, int &y, int w,
         getCascadePlace(frame, lastX, lastY, x, y, w, h);
     }
 #if 0  // !!!
-    if (frame->getLayer() == WinLayerDock) {
+    if (frame->doNotCover()) {
         if (frame->getState() & WinStateDockHorizontal)
             y = 0;
         else
@@ -914,7 +975,12 @@ void YWindowManager::getNewPosition(YFrameWindow *frame, int &x, int &y, int w,
 #endif
 }
 
-void YWindowManager::placeWindow(YFrameWindow *frame, int x, int y, int newClient, bool &canActivate) {
+void YWindowManager::placeWindow(YFrameWindow *frame, int x, int y,
+				 bool newClient, bool &
+#ifdef SM
+canActivate
+#endif
+) {
     YFrameClient *client = frame->client();
 
     int posWidth = client->width() + 2 * frame->borderX();
@@ -933,7 +999,7 @@ void YWindowManager::placeWindow(YFrameWindow *frame, int x, int y, int newClien
         WindowOption wo;
         frame->getWindowOptions(wo, true);
 
-        //printf("positioning %d %d %d %d %d\n", wo.gx, wo.gy, wo.gw, wo.gh, wo.gflags);
+        //msg("positioning %d %d %d %d %d", wo.gx, wo.gy, wo.gw, wo.gh, wo.gflags);
         if (wo.gh != 0 && wo.gw != 0) {
             if (wo.gflags & (WidthValue | HeightValue))
                 frame->setSize(wo.gw, wo.gh);
@@ -946,16 +1012,11 @@ void YWindowManager::placeWindow(YFrameWindow *frame, int x, int y, int newClien
 #endif
 
     if (newClient && client->adopted() && client->sizeHints() &&
-        (!(client->sizeHints()->flags & (USPosition | PPosition))
-         ||
+        (!(client->sizeHints()->flags & (USPosition | PPosition)) ||
          ((client->sizeHints()->flags & PPosition) &&
-          frame->frameOptions() & YFrameWindow::foIgnorePosition)
-        )
-       )
-    {
+          frame->frameOptions() & YFrameWindow::foIgnorePosition))) {
         getNewPosition(frame, x, y, posWidth, posHeight);
-        newClient = 0;
-    } else {
+        newClient = false;
     }
 
     int posX = x;
@@ -1039,7 +1100,7 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
         goto end;
     }
 
-    placeWindow(frame, cx, cy, (phase == phaseStartup) ? 0 : 1, canActivate);
+    placeWindow(frame, cx, cy, (phase != phaseStartup), canActivate);
 
 #ifdef SHAPE
     frame->setShape();
@@ -1063,7 +1124,7 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
             else
                 st = NormalState;
         }
-        MSG(("FRAME state = %d\n", st));
+        MSG(("FRAME state = %d", st));
         switch (st) {
         case IconicState:
             frame->setState(WinStateMinimized, WinStateMinimized);
@@ -1074,43 +1135,39 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
             break;
         }
     }
+
     if (frame->client()->getWinWorkspaceHint(&workspace))
         frame->setWorkspace(workspace);
 
-    if ((limitSize || limitPosition) && (phase != phaseStartup)) {
-        int posX = frame->x();
-        int posY = frame->y();
-        int posWidth = frame->width();
-        int posHeight = frame->height();
+    if ((limitSize || limitPosition) && 
+        (phase != phaseStartup) &&
+	!frame->doNotCover()) {
+        int posX(frame->x() + frame->borderX()),
+	    posY(frame->y() + frame->borderY()),
+	    posWidth(frame->width() - 2 * frame->borderX()),
+	    posHeight(frame->height() - 2 * frame->borderY());
 
-        posX += frame->borderX();
-        posY += frame->borderY();
-        posWidth -= 2 * frame->borderX();
-        posHeight -= 2 * frame->borderY();
         if (limitSize) {
-            int w = maxX(frame->getLayer()) - minX(frame->getLayer());
-            int h = maxY(frame->getLayer()) - minY(frame->getLayer());
-
-            if (posWidth > w) posWidth = w;
-            if (posHeight > h) posHeight = h;
+            posWidth = min(posWidth,
+	    		   maxX(frame->getLayer()) - minX(frame->getLayer()));
+            posHeight = min(posHeight,
+			    maxY(frame->getLayer()) - minY(frame->getLayer()));
 
             posHeight -= frame->titleY();
-            frame->client()->constrainSize(posWidth, posHeight, frame->getLayer(), 0);
+            frame->client()->constrainSize(posWidth, posHeight,
+	    				   frame->getLayer(), 0);
             posHeight += frame->titleY();
         }
-        if (limitPosition &&
-            !client->sizeHints() || !(client->sizeHints()->flags & USPosition))
-        {
-            if (posX + posWidth > int(maxX(frame->getLayer())))
-                posX = maxX(frame->getLayer()) - posWidth;
-            if (posY + posHeight > int(maxY(frame->getLayer())))
-                posY = maxY(frame->getLayer()) - posHeight;
-            if (posX < minX(frame->getLayer()))
-                posX = minX(frame->getLayer());
-            if (posY < minY(frame->getLayer()))
-                posY = minY(frame->getLayer());
 
+        if (limitPosition &&
+            !(client->sizeHints() &&
+	     (client->sizeHints()->flags & USPosition))) {
+            posX = clamp(posX, minX(frame->getLayer()),
+	    		       maxX(frame->getLayer()) - posWidth);
+            posY = clamp(posY, minY(frame->getLayer()),
+	    		       maxY(frame->getLayer()) - posHeight);
         }
+
         posX -= frame->borderX();
         posY -= frame->borderY();
         posWidth += 2 * frame->borderX();
@@ -1146,8 +1203,8 @@ YFrameWindow *YWindowManager::manageClient(Window win, bool mapClient) {
 #endif
     frame->updateNormalSize();
     frame->updateLayout();
-    if (frame->getLayer() == WinLayerDock)
-        updateWorkArea();
+    if (frame->doNotCover())
+	updateWorkArea();
 
     if (mapClient) {
         if (frame->getState() == 0 || frame->isRollup()) {
@@ -1223,7 +1280,7 @@ bool YWindowManager::focusTop(YFrameWindow *f) {
                       YFrameWindow::fwfSame |
                       YFrameWindow::fwfLayers |
                       YFrameWindow::fwfCycle);
-    //fprintf(stderr, "found focus %lX\n", f);
+    //msg("found focus %lX", f);
     if (!f) {
         setFocus(0);
         return false;
@@ -1234,15 +1291,15 @@ bool YWindowManager::focusTop(YFrameWindow *f) {
 
 YFrameWindow *YWindowManager::topLayer(long layer) {
     for (long l = layer; l >= 0; l--)
-        if (fTop[l] != 0)
-            return fTop[l];
+        if (fTop[l]) return fTop[l];
+
     return 0;
 }
 
 YFrameWindow *YWindowManager::bottomLayer(long layer) {
     for (long l = layer; l < WinLayerCount; l++)
-        if (fBottom[l] != 0)
-            return fBottom[l];
+        if (fBottom[l]) return fBottom[l];
+
     return 0;
 }
 
@@ -1350,7 +1407,7 @@ void YWindowManager::restackWindows(YFrameWindow *win) {
         XRestackWindows(app->display(), w, count);
     }
     if (i != count) {
-        fprintf(stderr, "i=%d, count=%d\n", i, count);
+        MSG(("i=%d, count=%d", i, count));
     }
     PRECONDITION(i == count);
     delete w;
@@ -1384,19 +1441,23 @@ int YWindowManager::maxY(long layer) const {
 }
 
 void YWindowManager::updateWorkArea() {
-    int nx1, ny1, nx2, ny2;
-    bool isHoriz;
+    int nMinX(0),
+	nMinY(0),
+    	nMaxX(width()),
+	nMaxY(height()),
+	midX((nMinX + nMaxX) / 2),
+	midY((nMinY + nMaxY) / 2);
 
-    nx1 = 0;
-    ny1 = 0;
-    nx2 = width();
-    ny2 = height();
+    YFrameWindow * w;
 
-    int midX = (nx1 + nx2) / 2;
-    int midY = (ny1 + ny2) / 2;
+    if (limitByDockLayer)
+	w = top(WinLayerDock);
+    else
+	for (w = topLayer();
+	     w && !(w->frameOptions() & YFrameWindow::foDoNotCover);
+	     w = w->nextLayer());
 
-    for (YFrameWindow *w = fTop[WinLayerDock]; w; w = w->next()) {
-        int anx1 = nx1, any1 = ny1, anx2 = nx2, any2 = ny2;
+    while(w) {
         // !!! FIX: WORKAREA windows must currently be sticky
 
         if (w->isHidden() ||
@@ -1404,50 +1465,56 @@ void YWindowManager::updateWorkArea() {
             w->isIconic() ||
             w->isMinimized() ||
             !w->visibleNow() ||
-            !w->isSticky())
-            continue;
+            !w->isSticky());
+else {
+      //      continue;
 
         // hack
-        isHoriz = false;
-        if (w->width() > w->height())
-            isHoriz = true;
+        int wMinX(nMinX), wMinY(nMinY), wMaxX(nMaxX), wMaxY(nMaxY);
+        bool const isHoriz(w->width() > w->height());
 
         if (!isHoriz /*!!!&& !(w->getState() & WinStateDockHorizontal)*/) {
             if (w->x() + int(w->width()) < midX)
-                anx1 = w->x() + w->width();
+                wMinX = w->x() + w->width();
             else if (w->x() > midX)
-                anx2 = w->x();
+                wMaxX = w->x();
         } else {
             if (w->y() + int(w->height()) < midY)
-                any1 = w->y() + w->height();
+                wMinY = w->y() + w->height();
             else if (w->y() > midY)
-                any2 = w->y();
+                wMaxY = w->y();
         }
-        if (anx1 > nx1) nx1 = anx1;
-        if (any1 > ny1) ny1 = any1;
-        if (anx2 < nx2) nx2 = anx2;
-        if (any2 < ny2) ny2 = any2;
-    }
-    if (fMinX != nx1 ||
-        fMinY != ny1 ||
-        fMaxX != nx2 ||
-        fMaxY != ny2)
-    {
-        int oldMinX = fMinX;
-        int oldMinY = fMinY;
 
-        fMinX = nx1;
-        fMinY = ny1;
-        fMaxX = nx2;
-        fMaxY = ny2;
+	nMinX = max(nMinX, wMinX);
+	nMinY = max(nMinY, wMinY);
+	nMaxX = min(nMaxX, wMaxX);
+	nMaxY = min(nMaxY, wMaxY);
+}
+	if (limitByDockLayer)
+	    w = w->next();
+	else
+	    do w = w->nextLayer();
+	    while (w && !(w->frameOptions() & YFrameWindow::foDoNotCover));
+    }
+
+    if (fMinX != nMinX || fMinY != nMinY || // -- store the new workarea ---
+        fMaxX != nMaxX || fMaxY != nMaxY) {
+
+        int const deltaX(nMinX - fMinX);
+        int const deltaY(nMinY - fMinY);
+
+        fMinX = nMinX; fMinY = nMinY;
+        fMaxX = nMaxX; fMaxY = nMaxY;
+
         if (fWorkAreaMoveWindows)
-            relocateWindows(fMinX - oldMinX, fMinY - oldMinY);
+            relocateWindows(deltaX, deltaY);
+
         resizeWindows();
-        putWorkArea();
+        announceWorkArea();
     }
 }
 
-void YWindowManager::putWorkArea() {
+void YWindowManager::announceWorkArea() {
     INT32 workArea[4];
 
     workArea[0] = minX(WinLayerNormal);
@@ -1463,46 +1530,29 @@ void YWindowManager::putWorkArea() {
 }
 
 void YWindowManager::relocateWindows(int dx, int dy) {
-    YFrameWindow *f = 0;
-
-    for (long l = WinLayerDock - 1; l > 0; l--)
-        if (fTop[l] != 0) {
-            f = fTop[l];
-            break;
-        }
-    if (f == 0)
-        return ;
-    while (f) {
-        f->setPosition(f->x() + dx, f->y() + dy);
-        f = f->nextLayer();
-    }
+    for (YFrameWindow * f(topLayer(WinLayerDock - 1)); f; f = f->nextLayer())
+	if (!f->doNotCover())
+	    f->setPosition(f->x() + dx, f->y() + dy);
 }
 
 void YWindowManager::resizeWindows() {
-    YFrameWindow *f = 0;
-
-    for (long l = WinLayerDock - 1; l > 0; l--)
-        if (fTop[l] != 0) {
-            f = fTop[l];
-            break;
-        }
-    if (f == 0)
-        return ;
-    while (f) {
-        if (f->isMaximized()) {
-            f->updateLayout();
-        }
+    for (YFrameWindow * f(topLayer(WinLayerDock - 1)); f; f = f->nextLayer())
+	if (!f->doNotCover()) {
+	    if (f->isMaximized() || f->canSize())
+		f->updateLayout();
+#if 0	
+            if (f->isMaximized())
+		f->updateLayout();
+#endif	
 #if 0
-        if (isMaximizedFully()) {
-            f->setGeometry(fMinX, fMinY, fMaxX - fMinX, fMaxY - fMinY);
-        } else if (f->isMaximizedVert()) {
-            f->setGeometry(f->x(), fMinY, f->width(), fMaxY - fMinY);
-        } else if (f->isMaximizedHoriz()) {
-            f->setGeometry(fMinX, f->y(), fMaxX - fMinX, f->height());
-        }
+	    if (isMaximizedFully())
+		f->setGeometry(fMinX, fMinY, fMaxX - fMinX, fMaxY - fMinY);
+	    else if (f->isMaximizedVert())
+		f->setGeometry(f->x(), fMinY, f->width(), fMaxY - fMinY);
+	    else if (f->isMaximizedHoriz())
+		f->setGeometry(fMinX, f->y(), fMaxX - fMinX, f->height());
 #endif
-        f = f->nextLayer();
-    }
+	}
 }
 
 void YWindowManager::activateWorkspace(long workspace) {
@@ -1515,6 +1565,7 @@ void YWindowManager::activateWorkspace(long workspace) {
             }
         }
 #endif
+        fLastWorkspace = fActiveWorkspace;
         fActiveWorkspace = workspace;
 #ifdef CONFIG_TASKBAR
         if (taskBar && taskBar->workspacesPane() &&
@@ -1536,26 +1587,21 @@ void YWindowManager::activateWorkspace(long workspace) {
 
         YFrameWindow *w;
 
-        w = bottomLayer();
-        while (w) {
+        for (w = bottomLayer(); w; w = w->prevLayer())
             if (!w->visibleNow()) {
                 w->updateState();
 #ifdef CONFIG_TASKBAR
                 w->updateTaskBar();
 #endif
             }
-            w = w->prevLayer();
-        }
-        w = topLayer();
-        while (w) {
+
+        for (w = topLayer(); w; w = w->nextLayer())
             if (w->visibleNow()) {
                 w->updateState();
 #ifdef CONFIG_TASKBAR
                 w->updateTaskBar();
 #endif
             }
-            w = w->nextLayer();
-        }
 
         if ((clickFocus || !strongPointerFocus)
             /* && (getFocus() == 0 || !getFocus()->visibleNow() || !getFocus()->isFocusable())*/)
@@ -1573,6 +1619,10 @@ void YWindowManager::activateWorkspace(long workspace) {
         if (taskBar && taskBar->taskPane())
             taskBar->taskPane()->relayout();
 #endif
+#ifndef LITE
+        if (workspaceSwitchStatus && (!showTaskBar || !taskBarShowWorkspaces))
+            statusWorkspace->begin(workspace);
+#endif
 #ifdef CONFIG_GUIEVENTS
         wmapp->signalGuiEvent(geWorkspaceChange);
 #endif
@@ -1587,15 +1637,10 @@ void YWindowManager::setWinWorkspace(long workspace) {
     activateWorkspace(workspace);
 }
 
-void YWindowManager::wmCloseSession() {
-    YFrameWindow *f = topLayer();
-
-    /* shutdown started */
-    while (f) {
+void YWindowManager::wmCloseSession() { // ----------------- shutdow started ---
+    for (YFrameWindow * f(topLayer()); f; f = f->nextLayer())
         if (f->client()->adopted()) // not to ourselves?
             f->wmClose();
-        f = f->nextLayer();
-    }
 }
 
 void YWindowManager::getIconPosition(YFrameWindow *frame, int *iconX, int *iconY) {
@@ -1702,18 +1747,18 @@ void YWindowManager::updateClientList() {
     int w, count = 0;
     XID *ids;
 
-    for (YFrameWindow *frame = topLayer(); frame; frame = frame->nextLayer())
-        if (frame->client() && frame->client()->adopted())
+    for (YFrameWindow * f(topLayer()); f; f = f->nextLayer())
+        if (f->client() && f->client()->adopted())
             count++;
 
     if ((ids = new XID[count]) == NULL)
         return ;
 
     w = 0;
-    for (YFrameWindow *frame2 = topLayer(); frame2; frame2 = frame2->nextLayer()) {
-        if (frame2->client() && frame2->client()->adopted())
-            ids[w++] = frame2->client()->handle();
-    }
+    for (YFrameWindow * f(topLayer()); f; f = f->nextLayer())
+        if (f->client() && f->client()->adopted())
+            ids[w++] = f->client()->handle();
+
     PRECONDITION(w == count);
     XChangeProperty(app->display(), desktop->handle(),
                     _XA_WIN_CLIENT_LIST,
@@ -1727,10 +1772,10 @@ void YWindowManager::updateClientList() {
 void YWindowManager::checkLogout() {
     if (fShuttingDown && !haveClients()) {
         if (rebootOrShutdown == 1 && rebootCommand && rebootCommand[0]) {
-            printf("icewm: reboot... (%s)\n", rebootCommand);
+            msg("reboot... (%s)", rebootCommand);
             system(rebootCommand);
         } else if (rebootOrShutdown == 2 && shutdownCommand && shutdownCommand[0]) {
-            printf("icewm: shutdown ... (%s)\n", shutdownCommand);
+            msg("shutdown ... (%s)", shutdownCommand);
             system(shutdownCommand);
         }
         app->exit(0);
@@ -1751,8 +1796,8 @@ void YWindowManager::removeClientFrame(YFrameWindow *frame) {
         setFocus(0);
     if (colormapWindow() == frame)
         setColormapWindow(getFocus());
-    if (frame->getLayer() == WinLayerDock)
-        updateWorkArea();
+    if (frame->doNotCover())
+	updateWorkArea();
 }
 
 void YWindowManager::switchFocusTo(YFrameWindow *frame) {
@@ -1760,7 +1805,7 @@ void YWindowManager::switchFocusTo(YFrameWindow *frame) {
         if (fFocusWin)
             fFocusWin->loseWinFocus();
         fFocusWin = frame;
-        ///printf("setting %lX\n", fFocusWin);
+        ///msg("setting %lX", fFocusWin);
         if (fFocusWin)
             fFocusWin->setWinFocus();
     }
@@ -1769,7 +1814,7 @@ void YWindowManager::switchFocusTo(YFrameWindow *frame) {
 void YWindowManager::switchFocusFrom(YFrameWindow *frame) {
     if (frame == fFocusWin) {
         if (fFocusWin) {
-            ///printf("losing %lX\n", fFocusWin);
+            ///msg("losing %lX", fFocusWin);
             fFocusWin->loseWinFocus();
         }
         fFocusWin = 0;
@@ -1841,6 +1886,10 @@ void YWindowManager::switchToNextWorkspace(bool takeCurrent) {
     switchToWorkspace(nw, takeCurrent);
 }
 
+void YWindowManager::switchToLastWorkspace(bool takeCurrent) {
+    switchToWorkspace(lastWorkspace(), takeCurrent);
+}
+
 void YWindowManager::tilePlace(YFrameWindow *w, int tx, int ty, int tw, int th) {
     w->setState(WinStateMinimized |
                 WinStateRollup |
@@ -1990,14 +2039,10 @@ void YWindowManager::undoArrange() {
 }
 
 bool YWindowManager::haveClients() {
-    YFrameWindow *f = topLayer();
-
-    for (; f ; f = f->nextLayer()) {
-        if (!f->canClose())
-            continue;
-        if (f->client()->adopted())
+    for (YFrameWindow * f(topLayer()); f ; f = f->nextLayer())
+        if (f->canClose() && f->client()->adopted())
             return true;
-    }
+
     return false;
 }
 
@@ -2020,7 +2065,7 @@ EdgeSwitch::EdgeSwitch(YWindowManager *manager, int delta): YWindow(manager) {
         shape = XC_sb_right_arrow;
 
     cursor = XCreateFontCursor(app->display(), shape);
-    setPointer(leftPointer);
+    setPointer(YApplication::leftPointer);
 }
 
 EdgeSwitch::~EdgeSwitch() {
@@ -2056,6 +2101,6 @@ bool EdgeSwitch::handleTimer(YTimer *t) {
         fManager->switchToPrevWorkspace(false);
     else
         fManager->switchToNextWorkspace(false);
-    setPointer(leftPointer);
+    setPointer(YApplication::leftPointer);
     return false;
 }
