@@ -65,6 +65,10 @@ public:
     void manageClients();
     void unmanageClients();
 
+    Window findWindow(char const * resource);
+    Window findWindow(Window root, char const * wmInstance,
+    		      char const * wmClass);
+
     YFrameWindow *findFrame(Window win);
     YFrameClient *findClient(Window win);
     YFrameWindow *manageClient(Window win, bool mapClient = false);
@@ -99,7 +103,7 @@ public:
                   int &px, int &py, int &cover);
     bool getSmartPlace(bool down, YFrameWindow *frame, int &x, int &y, int w, int h);
     void getNewPosition(YFrameWindow *frame, int &x, int &y, int w, int h);
-    void placeWindow(YFrameWindow *frame, int x, int y, int newClient, bool &canActivate);
+    void placeWindow(YFrameWindow *frame, int x, int y, bool newClient, bool &canActivate);
 
     YFrameWindow *top(long layer) const { return fTop[layer]; }
     void setTop(long layer, YFrameWindow *top);
@@ -122,11 +126,12 @@ public:
 #endif
 
     long activeWorkspace() const { return fActiveWorkspace; }
+    long lastWorkspace() const { return fLastWorkspace; }
     void activateWorkspace(long workspace);
     long workspaceCount() const { return ::workspaceCount; }
     const char *workspaceName(long workspace) const { return ::workspaceNames[workspace]; }
 
-    void putWorkArea();
+    void announceWorkArea();
     void setWinWorkspace(long workspace);
     void updateWorkArea();
     void resizeWindows();
@@ -150,6 +155,7 @@ public:
     void switchToWorkspace(long nw, bool takeCurrent);
     void switchToPrevWorkspace(bool takeCurrent);
     void switchToNextWorkspace(bool takeCurrent);
+    void switchToLastWorkspace(bool takeCurrent);
 
     void tilePlace(YFrameWindow *w, int tx, int ty, int tw, int th);
     void tileWindows(YFrameWindow **w, int count, bool vertical);
@@ -179,6 +185,7 @@ private:
     YFrameWindow *fTop[WinLayerCount];
     YFrameWindow *fBottom[WinLayerCount];
     long fActiveWorkspace;
+    long fLastWorkspace;
     YFrameWindow *fColormapWindow;
     int fMinX, fMinY, fMaxX, fMaxY;
     EdgeSwitch *fLeftSwitch, *fRightSwitch;
