@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 
@@ -14,6 +14,8 @@
 #include "sysdep.h"
 #include "base.h"
 
+#include "intl.h"
+
 char *winOptFile = 0;
 WindowOptions *defOptions = 0;
 WindowOptions *hintOptions = 0;
@@ -91,7 +93,7 @@ WindowOption *WindowOptions::getWindowOption(const char *name, bool create, bool
 void WindowOptions::setWinOption(const char *class_instance, const char *opt, const char *arg) {
     WindowOption *op = getWindowOption(class_instance, true);
 
-    //fprintf(stderr, "%s-%s-%s\\", class_instance, opt, arg);
+    //msg("%s-%s-%s", class_instance, opt, arg);
 
     if (strcmp(opt, "icon") == 0) {
         op->icon = newstr(arg);
@@ -106,7 +108,7 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
         op->gw = 0;
         op->gh = 0;
 
-        //printf("parsing %s\n", arg);
+        //msg("parsing %s", arg);
         if ((op->gflags = XParseGeometry(arg, &rx, &ry, &rw, &rh)) != 0) {
             if (op->gflags & XNegative)
                 rx = desktop->width() - rx;
@@ -116,7 +118,7 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
             op->gy = ry;
             op->gw = rw;
             op->gh = rh;
-            //printf("parsed %d %d %d %d %d\n", rx, ry, rw, rh, op->gflags);
+            //msg("parsed %d %d %d %d %d", rx, ry, rw, rh, op->gflags);
         }
     } else if (strcmp(opt, "layer") == 0) {
         char *endptr;
@@ -173,7 +175,8 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
             { 2, "fullKeys", YFrameWindow::foFullKeys }, //
             { 2, "noFocusOnAppRaise", YFrameWindow::foNoFocusOnAppRaise }, //
             { 2, "ignoreNoFocusHint", YFrameWindow::foIgnoreNoFocusHint }, //
-            { 2, "ignorePositionHint", YFrameWindow::foIgnorePosition } //
+            { 2, "ignorePositionHint", YFrameWindow::foIgnorePosition }, //
+            { 2, "doNotCover", YFrameWindow::foDoNotCover } //
         };
         int t = (atoi(arg) == 1) ? 1 : 0;
 
@@ -190,7 +193,7 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
                 what = &op->options;
                 what_mask = &op->option_mask;
             } else {
-                fprintf(stderr, "error in window option: %s\n", opt);
+                msg(_("Error in window option: %s"), opt);
                 break;
             }
 
@@ -203,7 +206,7 @@ void WindowOptions::setWinOption(const char *class_instance, const char *opt, co
                 return ;
             }
         }
-        fprintf(stderr, "unknown window option: %s\n", opt);
+        msg(_("Unknown window option: %s"), opt);
     }
 }
 
@@ -262,7 +265,7 @@ char *parseWinOptions(char *data) {
         if (e == w || *p == 0)
             break;
         if (c == 0) {
-            fprintf(stderr, "icewm: syntax error in window options\n");
+            msg(_("Syntax error in window options"));
             break;
         }
 
@@ -300,7 +303,7 @@ char *parseWinOptions(char *data) {
     }
     return p;
 nomem:
-    fprintf(stderr, "icewm: out of memory for window options\n");
+    msg(_("Out of memory for window options"));
     return 0;
 }
 
