@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  *
  * Windows/OS2 like Alt{+Shift}+Tab window switching
  */
@@ -137,6 +137,7 @@ YFrameWindow *SwitchWindow::nextWindow(YFrameWindow *from, bool zdown, bool next
     if (n == 0)
         n = from->findWindow(flags |
                              YFrameWindow::fwfVisible |
+                             (quickSwitchToMinimized) ? 0 : YFrameWindow::fwfUnminimized |
                              YFrameWindow::fwfSame);
     if (n == 0)
         n = fLastWindow;
@@ -172,8 +173,14 @@ void SwitchWindow::cancel() {
     if (fLastWindow) {
         displayFocus(fLastWindow);
     } else if (fActiveWindow) {
-        fActiveWindow->wmRaise();
-        manager->activate(fActiveWindow, true);
+        if (!quickSwitchToMinimized &&
+            fActiveWindow->isMinimized())
+        {
+            // !!! workaround, because the window is actually focused...
+        } else {
+            manager->activate(fActiveWindow, true);
+            fActiveWindow->wmRaise();
+        }
     }
     fLastWindow = fActiveWindow = 0;
 }
@@ -182,12 +189,18 @@ void SwitchWindow::accept() {
     if (fActiveWindow == 0)
         cancel();
     else {
-        manager->activate(fActiveWindow, true);
+        if (!quickSwitchToMinimized &&
+            fActiveWindow->isMinimized())
+        {
+            // !!! workaround, because the window is actually focused...
+        } else {
+            manager->activate(fActiveWindow, true);
+            fActiveWindow->wmRaise();
+        }
         if (isUp) {
             cancelPopup();
             isUp = false;
         }
-        fActiveWindow->wmRaise();
         //manager->activate(fActiveWindow, true);
     }
     fLastWindow = fActiveWindow = 0;
@@ -210,13 +223,14 @@ void SwitchWindow::destroyedFrame(YFrameWindow *frame) {
 bool SwitchWindow::handleKey(const XKeyEvent &key) {
     KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
     unsigned int m = KEY_MODMASK(key.state);
+    unsigned int vm = VMod(m);
 
     if (key.type == KeyPress) {
-        if ((k == XK_Tab) && !(m & ShiftMask)) {
+        if ((IS_WMKEY(k, vm, gKeySysSwitchNext))) {
             fActiveWindow = nextWindow(fActiveWindow, true, true);
             displayFocus(fActiveWindow);
             return true;
-        } else if ((/*k == XK_BackTab ||*/ k == XK_Tab) && (m & ShiftMask)) {
+        } else if ((IS_WMKEY(k, vm, gKeySysSwitchLast))) {
             fActiveWindow = nextWindow(fActiveWindow, false, true);
             displayFocus(fActiveWindow);
             return true;
@@ -224,12 +238,12 @@ bool SwitchWindow::handleKey(const XKeyEvent &key) {
             cancel();
             return true;
         }
-        if (k == XK_Tab && !modDown(m)) {
+        if ((IS_WMKEY(k, vm, gKeySysSwitchNext)) && !modDown(m)) {
             accept();
             return true;
         }
     } else if (key.type == KeyRelease) {
-        if (k == XK_Tab && !modDown(m)) {
+        if ((IS_WMKEY(k, vm, gKeySysSwitchNext)) && !modDown(m)) {
             accept();
             return true;
         } else if (isModKey(key.keycode)) {
