@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,98 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  *
  * TaskBar
  */
@@ -34,6 +34,8 @@
 
 #include "aapm.h"
 
+#include "intl.h"
+
 YColor *taskBarBg = 0;
 
 YTimer *TaskBarApp::fRaiseTimer = 0;
@@ -86,7 +88,7 @@ static void initPixmaps() {
     loadPixmap(cpaths, base, "taskbuttonactive.xpm", &taskbuttonactivePixmap);
     loadPixmap(cpaths, base, "taskbuttonminimized.xpm", &taskbuttonminimizedPixmap);
 
-#ifdef CONFIG_MAILBOX
+#ifdef CONFIG_APPLET_MAILBOX
     base = "mailbox/";
     memcpy(cpaths, paths, sizeof(cpaths));
     verifyPaths(cpaths, base);
@@ -97,7 +99,7 @@ static void initPixmaps() {
     loadPixmap(cpaths, base, "newmail.xpm", &newMailPixmap);
 #endif
 
-#ifdef CONFIG_CLOCK
+#ifdef CONFIG_APPLET_CLOCK
     base = "ledclock/";
     memcpy(cpaths, paths, sizeof(cpaths));
     verifyPaths(cpaths, base);
@@ -140,15 +142,19 @@ YWindow(aParent)
     initPixmaps();
 
 #if 1
-    setWindowTitle("Task Bar");
-    setIconTitle("Task Bar");
+    setWindowTitle(_("Task Bar"));
+    setIconTitle(_("Task Bar"));
     setWinStateHint(WinStateAllWorkspaces, WinStateAllWorkspaces);
     //!!!setWinStateHint(WinStateDockHorizontal, WinStateDockHorizontal);
+    setWinHintsHint(WinHintsSkipFocus |
+    		    WinHintsSkipWindowMenu |
+    		    WinHintsSkipTaskBar |
+		    (taskBarAutoHide ? 0 : WinHintsDoNotCover));
+    
     setWinWorkspaceHint(0);
-    if (taskBarAutoHide)
-        setWinLayerHint(WinLayerAboveDock);
-    else
-        setWinLayerHint(WinLayerDock);
+    setWinLayerHint(taskBarAutoHide ? WinLayerAboveDock :
+		    taskBarKeepBelow ? WinLayerBelow : WinLayerDock);
+
     {
         XWMHints wmh;
 
@@ -187,6 +193,7 @@ YWindow(aParent)
                         32, PropModeReplace,
                         (unsigned char *)arg, 2);
     }
+    setPointer(YApplication::leftPointer);
     setDND(true);
 
     fAutoHideTimer = new YTimer(autoHideDelay);
@@ -197,41 +204,41 @@ YWindow(aParent)
     taskBarMenu = new YMenu();
     if (taskBarMenu) {
         taskBarMenu->setActionListener(this);
-        taskBarMenu->addItem("Tile Vertically", 5, "", actionTileVertical);
-        taskBarMenu->addItem("Tile Horizontally", 1, "", actionTileHorizontal);
-        taskBarMenu->addItem("Cascade", 1, "", actionCascade);
-        taskBarMenu->addItem("Arrange", 1, "", actionArrange);
-        taskBarMenu->addItem("Minimize All", 0, "", actionMinimizeAll);
-        taskBarMenu->addItem("Hide All", 0, "", actionHideAll);
-        taskBarMenu->addItem("Undo", 1, "", actionUndoArrange);
+        taskBarMenu->addItem(_("Tile _Vertically"), -2, "", actionTileVertical);
+        taskBarMenu->addItem(_("T_ile Horizontally"), -2, "", actionTileHorizontal);
+        taskBarMenu->addItem(_("Ca_scade"), -2, "", actionCascade);
+        taskBarMenu->addItem(_("_Arrange"), -2, "", actionArrange);
+        taskBarMenu->addItem(_("_Minimize All"), -2, "", actionMinimizeAll);
+        taskBarMenu->addItem(_("_Hide All"), -2, "", actionHideAll);
+        taskBarMenu->addItem(_("_Undo"), -2, "", actionUndoArrange);
         if (minimizeToDesktop)
-            taskBarMenu->addItem("Arrange Icons", 8, "", actionArrangeIcons)->setEnabled(false);
+            taskBarMenu->addItem(_("Arrange _Icons"), -2, "", actionArrangeIcons)->setEnabled(false);
         taskBarMenu->addSeparator();
 #ifdef CONFIG_WINMENU
-        taskBarMenu->addItem("Windows", 0, actionWindowList, windowListMenu);
+        taskBarMenu->addItem(_("_Windows"), -2, actionWindowList, windowListMenu);
 #endif
         taskBarMenu->addSeparator();
-        taskBarMenu->addItem("Refresh", 0, "", actionRefresh);
+        taskBarMenu->addItem(_("_Refresh"), -2, "", actionRefresh);
 
 #ifndef LITE
 #if 0
         YMenu *helpMenu; // !!!
 
         helpMenu = new YMenu();
-        helpMenu->addItem("License", 0, "", actionLicense);
+        helpMenu->addItem(_("_License"), -2, "", actionLicense);
         helpMenu->addSeparator();
-        helpMenu->addItem("About", 0, "", actionAbout);
+        helpMenu->addItem(_("_About"), -2, "", actionAbout);
 #endif
 
-        taskBarMenu->addItem("About", 0, actionAbout, 0);
+        taskBarMenu->addItem(_("_About"), -2, actionAbout, 0);
 #endif
         taskBarMenu->addSeparator();
-        taskBarMenu->addItem("Logout...", 0, actionLogout, logoutMenu);
+        taskBarMenu->addItem(_("_Logout..."), -2, actionLogout, logoutMenu);
     }
 
     fAddressBar = 0;
 
-#ifdef CONFIG_CPUSTATUS
+#ifdef CONFIG_APPLET_CPU_STATUS
 #if (defined(linux) || defined(HAVE_KSTAT_H))
     if (taskBarShowCPUStatus)
         fCPUStatus = new CPUStatus(cpuCommand, this);
@@ -252,15 +259,15 @@ YWindow(aParent)
         if (fClock->height() + ADD1 > ht) ht = fClock->height() + ADD1;
     } else
         fClock = 0;
-#ifdef CONFIG_APM
+#ifdef CONFIG_APPLET_APM
     if (taskBarShowApm && access("/proc/apm", 0) == 0) {
-	fApm = new YApm(this);
-	if (fApm->height() + ADD1 > ht) ht = fApm->height() + ADD1;
+        fApm = new YApm(this);
+        if (fApm->height() + ADD1 > ht) ht = fApm->height() + ADD1;
     } else
         fApm = 0;
 #endif
 
-#ifdef CONFIG_MAILBOX
+#ifdef CONFIG_APPLET_MAILBOX
     if (taskBarShowMailboxStatus)
         fMailBoxStatus = new MailBoxStatus(mailBoxPath, mailCommand, this);
     else
@@ -308,7 +315,7 @@ YWindow(aParent)
 
         leftX = 2;
         rightX = width() - 4;
-#ifdef CONFIG_CLOCK
+#ifdef CONFIG_APPLET_CLOCK
         if (fClock) {
             fClock->setPosition(rightX - fClock->width(),
                                 BASE1 + (ht - ADD1 - fClock->height()) / 2);
@@ -316,15 +323,15 @@ YWindow(aParent)
             rightX -= fClock->width() + 2;
         }
 #endif
-#ifdef CONFIG_APM
+#ifdef CONFIG_APPLET_APM
         if (fApm) {
             rightX -= 2;
             fApm->setPosition(rightX - fApm->width(), BASE1 + (ht - ADD1 - fApm->height()) / 2);
-	    fApm->show();
-	    rightX -= fApm->width() + 2;
-	}    
+            fApm->show();
+            rightX -= fApm->width() + 2;
+        }
 #endif
-#ifdef CONFIG_MAILBOX
+#ifdef CONFIG_APPLET_MAILBOX
         if (fMailBoxStatus) {
             fMailBoxStatus->setPosition(rightX - fMailBoxStatus->width() - 1,
                                         BASE2 + (ht - ADD2 - fMailBoxStatus->height()) / 2);
@@ -332,7 +339,7 @@ YWindow(aParent)
             rightX -= fMailBoxStatus->width() + 2;
         }
 #endif
-#ifdef CONFIG_CPUSTATUS
+#ifdef CONFIG_APPLET_CPU_STATUS
 #if (defined(linux) || defined(HAVE_KSTAT_H))
         if (fCPUStatus) {
             fCPUStatus->setPosition(rightX - fCPUStatus->width() - 1,
@@ -376,18 +383,20 @@ YWindow(aParent)
         }
 #endif
 
+        if (showAddressBar) {
 #ifdef CONFIG_ADDRESSBAR
-        fAddressBar = new AddressBar(this);
-        if (fAddressBar) {
-            leftX += 2;
-            fAddressBar->setGeometry(leftX,
-                                     BASE1 + (ht - ADD1 - fAddressBar->height()) / 2,
-                                     rightX - leftX - 4,
-                                     fAddressBar->height());
+            fAddressBar = new AddressBar(this);
+            if (fAddressBar) {
+                leftX += 2;
+                fAddressBar->setGeometry(leftX,
+                                         BASE1 + (ht - ADD1 - fAddressBar->height()) / 2,
+                                         rightX - leftX - 4,
+                                         fAddressBar->height());
 
-            fAddressBar->show();
-        }
+                fAddressBar->show();
+            }
 #endif
+        }
 
         leftX = 2;
         rightX = width() - 4;
@@ -406,7 +415,7 @@ YWindow(aParent)
 
         leftX = 2;
         rightX = width() - 4;
-#ifdef CONFIG_CLOCK
+#ifdef CONFIG_APPLET_CLOCK
         if (fClock) {
             fClock->setPosition(rightX - fClock->width(),
                                 BASE1 + (ht - ADD1 - fClock->height()) / 2);
@@ -414,7 +423,7 @@ YWindow(aParent)
             rightX -= fClock->width() + 2;
         }
 #endif
-#ifdef CONFIG_MAILBOX
+#ifdef CONFIG_APPLET_MAILBOX
         if (fMailBoxStatus) {
             fMailBoxStatus->setPosition(rightX - fMailBoxStatus->width() - 1,
                                         BASE2 + (ht - ADD2 - fMailBoxStatus->height()) / 2);
@@ -422,7 +431,7 @@ YWindow(aParent)
             rightX -= fMailBoxStatus->width() + 2;
         }
 #endif
-#ifdef CONFIG_CPUSTATUS
+#ifdef CONFIG_APPLET_CPU_STATUS
 #if (defined(linux) || defined(HAVE_KSTAT_H))
         if (fCPUStatus) {
             fCPUStatus->setPosition(rightX - fCPUStatus->width() - 1,
@@ -441,7 +450,7 @@ YWindow(aParent)
             rightX -= fNetStatus->width() + 2;
         }
 #endif
-#ifdef CONFIG_APM
+#ifdef CONFIG_APPLET_APM
         if (fApm) {
             rightX -= 2;
             fApm->setPosition(rightX - fApm->width(), BASE1 + (ht - ADD1 - fApm->height()) / 2);
@@ -498,7 +507,7 @@ YWindow(aParent)
     } else {
         fTasks = 0;
 #ifdef CONFIG_ADDRESSBAR
-        if (fAddressBar == 0) {
+        if (showAddressBar && fAddressBar == 0) {
             fAddressBar = new AddressBar(this);
             if (fAddressBar) {
                 leftX += 2;
@@ -522,10 +531,10 @@ TaskBar::~TaskBar() {
         fAutoHideTimer->setTimerListener(0);
         delete fAutoHideTimer; fAutoHideTimer = 0;
     }
-#ifdef CONFIG_CLOCK
+#ifdef CONFIG_APPLET_CLOCK
     delete fClock; fClock = 0;
 #endif
-#ifdef CONFIG_MAILBOX
+#ifdef CONFIG_APPLET_MAILBOX
     delete fMailBoxStatus; fMailBoxStatus = 0;
 #endif
     delete fApplications; fApplications = 0;
@@ -540,14 +549,14 @@ TaskBar::~TaskBar() {
     delete taskbuttonminimizedPixmap;
     delete startPixmap;
     delete windowsPixmap;
-#ifdef CONFIG_MAILBOX
+#ifdef CONFIG_APPLET_MAILBOX
     delete mailPixmap;
     delete noMailPixmap;
     delete errMailPixmap;
     delete unreadMailPixmap;
     delete newMailPixmap;
 #endif
-#ifdef CONFIG_CLOCK
+#ifdef CONFIG_APPLET_CLOCK
     delete PixSpace;
     delete PixSlash;
     delete PixDot;
@@ -557,7 +566,7 @@ TaskBar::~TaskBar() {
     delete PixColon;
     for (int n = 0; n < 10; n++) delete PixNum[n];
 #endif
-#ifdef CONFIG_APM
+#ifdef CONFIG_APPLET_APM
     delete fApm; fApm = 0;
 #endif
     taskBar = 0;
@@ -745,10 +754,8 @@ void TaskBar::showBar(bool visible) {
         if (getFrame() == 0)
             manager->mapClient(handle());
         if (getFrame() != 0) {
-            if (taskBarAutoHide)
-                getFrame()->setLayer(WinLayerAboveDock);
-            else
-                getFrame()->setLayer(WinLayerDock);
+	    setWinLayerHint(taskBarAutoHide ? WinLayerAboveDock :
+			    taskBarKeepBelow ? WinLayerBelow : WinLayerDock);
             getFrame()->setState(WinStateAllWorkspaces, WinStateAllWorkspaces);
             getFrame()->activate(true);
             updateLocation();
@@ -760,6 +767,6 @@ void TaskBar::actionPerformed(YAction *action, unsigned int modifiers) {
     wmapp->actionPerformed(action, modifiers);
 }
 
-void TaskBar::handlePopDown(YPopupWindow *popup) {
+void TaskBar::handlePopDown(YPopupWindow */*popup*/) {
 }
 #endif
