@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1998 Marko Macek
+ * Copyright (C) 1998-2001 Marko Macek
  */
 #include "config.h"
 #include "yfull.h"
@@ -14,6 +14,8 @@
 #include "MwmUtil.h"
 #include "prefs.h"
 
+#include "intl.h"
+
 #ifdef SM
 #include <X11/SM/SMlib.h>
 #endif
@@ -65,9 +67,9 @@ Atom XA_XdndFinished;
 
 Colormap defaultColormap;
 
-Cursor leftPointer;
-Cursor rightPointer;
-Cursor movePointer;
+YCursor YApplication::leftPointer;
+YCursor YApplication::rightPointer;
+YCursor YApplication::movePointer;
 
 YColor *YColor::black = 0;
 YColor *YColor::white = 0;
@@ -121,7 +123,7 @@ void iceWatchFD(IceConn conn,
 {
     if (opening) {
         if (IceSMfd != -1) { // shouldn't happen
-            fprintf(stderr, "TOO MANY ICE CONNECTIONS -- not supported\n");
+            warn(_("TOO MANY ICE CONNECTIONS -- not supported"));
         } else {
             IceSMfd = IceConnectionNumber(conn);
             fcntl(IceSMfd, F_SETFD, FD_CLOEXEC);
@@ -179,6 +181,12 @@ static void setSMProperties() {
     };
 
     char *user = getenv("USER");
+    if (!user) // not a user?
+        user = getenv("LOGNAME");
+    if (!user) {
+        msg(_("$USER or $LOGNAME not set?"));
+        return ;
+    }
     const char *clientId = "-clientId";
 
     programVal.length = strlen(sessionProg);
@@ -213,7 +221,7 @@ static void initSM() {
     if (getenv("SESSION_MANAGER") == 0)
         return;
     if (IceAddConnectionWatch(&iceWatchFD, NULL) == 0) {
-        fprintf(stderr, "IceAddConnectionWatch failed.");
+        warn(_("Session Manager: IceAddConnectionWatch failed."));
         return ;
     }
 
@@ -241,7 +249,7 @@ static void initSM() {
                                     oldSessionId, &newSessionId,
                                     sizeof(error_str), error_str)) == NULL)
     {
-        fprintf(stderr, "session mgr init error: %s\n", error_str);
+        warn(_("Session Manager: Init error: %s"), error_str);
         return ;
     }
     IceSMconn = SmcGetIceConnection(SMconn);
@@ -375,11 +383,14 @@ private:
 };
 
 void initSignals() {
+    sigemptyset(&signalMask);
+    sigaddset(&signalMask, SIGHUP);
+    sigprocmask(SIG_BLOCK, &signalMask, &oldSignalMask);
     sigemptyset(&signalMask);
     sigprocmask(SIG_BLOCK, &signalMask, &oldSignalMask);
 
     if (pipe(signalPipe) != 0)
-        die(2, "pipe create failed, errno=%d.", errno);
+        die(2, _("Pipe creation failed (errno=%d)."), errno);
     fcntl(signalPipe[1], F_SETFL, O_NONBLOCK);
 }
 
@@ -443,9 +454,9 @@ static void initAtoms() {
 }
 
 static void initPointers() {
-    leftPointer = XCreateFontCursor(app->display(), XC_left_ptr);
-    rightPointer = XCreateFontCursor(app->display(), XC_right_ptr);
-    movePointer = XCreateFontCursor(app->display(), XC_fleur);
+    YApplication::leftPointer.load("left.xpm",  XC_left_ptr);
+    YApplication::rightPointer.load("right.xpm", XC_right_ptr);
+    YApplication::movePointer.load("move.xpm",  XC_fleur);
 }
 
 static void initColors() {
@@ -568,7 +579,9 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
     }
 
     if (!(fDisplay = XOpenDisplay(displayName)))
-        die(1, "Can't open display: %s. X must be running and $DISPLAY set.", displayName ? displayName : "<none>");
+        die(1, _("Can't open display: %s. "
+                 "X must be running and $DISPLAY set."),
+            displayName ? displayName : _("<none>"));
 
     if (sync)
         XSynchronize(display(), True);
@@ -577,6 +590,8 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
 
     windowContext = XUniqueContext();
 
+    new YDesktop(0, RootWindow(display(), DefaultScreen(display())));
+
     initSignals();
     initAtoms();
     initModifiers();
@@ -588,14 +603,12 @@ YApplication::YApplication(int *argc, char ***argv, const char *displayName) {
                                            &shapeEventBase, &shapeErrorBase);
 #endif
 
-    new YDesktop(0, RootWindow(display(), DefaultScreen(display())));
-
 #ifndef LITE
     initIcons();
 #endif
 
 #ifdef SM
-    sessionProg = (*argv)[0]; //"icewm";
+    sessionProg = (*argv)[0]; //ICEWMEXE;
     initSM();
 #endif
 
@@ -669,7 +682,10 @@ void YApplication::getTimeout(struct timeval *timeout) {
         }
         t = t->fNext;
     }
-    if (timercmp(&curtime, timeout, >)) {
+    if ((curtime.tv_sec == timeout->tv_sec &&
+         curtime.tv_usec == timeout->tv_usec)
+        || timercmp(&curtime, timeout, >))
+    {
         timeout->tv_sec = 0;
         timeout->tv_usec = 1;
     } else {
@@ -680,7 +696,7 @@ void YApplication::getTimeout(struct timeval *timeout) {
             timeout->tv_sec--;
         }
     }
-    //printf("set: %d %d\n", timeout->tv_sec, timeout->tv_usec);
+    //msg("set: %d %d", timeout->tv_sec, timeout->tv_usec);
     PRECONDITION(timeout->tv_sec >= 0);
     PRECONDITION(timeout->tv_usec >= 0);
 }
@@ -742,7 +758,7 @@ int YApplication::mainLoop() {
 
             XNextEvent(display(), &xev);
             xeventcount++;
-            //printf("%d\n", xev.type);
+            //msg("%d", xev.type);
 
             saveEventTime(xev);
 
@@ -783,10 +799,9 @@ int YApplication::mainLoop() {
                         if (xev.type == MapRequest) {
                             // !!! java seems to do this ugliness
                             //YFrameWindow *f = getFrame(xev.xany.window);
-                            fprintf(stderr,
-                                    "BUG? mapRequest for window %lX sent to destroyed frame %lX!\n",
-                                    xev.xmaprequest.parent,
-                                    xev.xmaprequest.window);
+                            msg("BUG? mapRequest for window %lX sent to destroyed frame %lX!",
+                                xev.xmaprequest.parent,
+                                xev.xmaprequest.window);
                             desktop->handleEvent(xev);
                         } else if (xev.type != DestroyNotify) {
                             MSG(("unknown window 0x%lX event=%d", xev.xany.window, xev.type));
@@ -866,7 +881,7 @@ int YApplication::mainLoop() {
                 handleTimeouts();
             } else if (rc == -1) {
                 if (errno != EINTR)
-                    fprintf(stderr, "select: errno=%d\n", errno);
+                    warn(_("Message Loop: select failed (errno=%d)"), errno);
             } else {
             if (signalPipe[0] != -1) {
                 if (FD_ISSET(signalPipe[0], &read_fds)) {
@@ -1191,6 +1206,7 @@ void YApplication::initModifiers() {
         KeyCode metaKeyCode = sym2code(XK_Meta_L);
         KeyCode superKeyCode = sym2code(XK_Super_L);
         KeyCode hyperKeyCode = sym2code(XK_Hyper_L);
+        KeyCode modeSwitchCode = sym2code(XK_Mode_switch);
 
         // A keyboard might lack Alt_L but have Alt_R.
         if (!altKeyCode)
@@ -1221,18 +1237,17 @@ void YApplication::initModifiers() {
                     SuperMask = (1 << m);
                 if (*c == hyperKeyCode)
                     HyperMask = (1 << m);
+                if (*c == modeSwitchCode)
+                    ModeSwitchMask = (1 << m);
             }
+
+	XFreeModifiermap(xmk);
     }
     if (MetaMask == AltMask)
         MetaMask = 0;
-#ifdef DEBUG
-    if (debug)
-        fprintf(stderr, "alt:%d meta:%d num:%d scroll:%d\n",
-                AltMask,
-                MetaMask,
-                NumLockMask,
-                ScrollLockMask);
-#endif
+
+    MSG(("alt:%d meta:%d num:%d scroll:%d",
+         AltMask, MetaMask, NumLockMask, ScrollLockMask));
 
     // some hacks for "broken" modifier configurations
 
@@ -1266,7 +1281,8 @@ void YApplication::initModifiers() {
         AltMask |
         MetaMask |
         SuperMask |
-        HyperMask;
+        HyperMask |
+	ModeSwitchMask;
 
     ButtonMask =
         Button1Mask |
@@ -1282,6 +1298,9 @@ void YApplication::runProgram(const char *str, const char *const *args) {
     XSync(app->display(), False);
     if (fork() == 0) {
         app->resetSignals();
+        sigemptyset(&signalMask);
+        sigaddset(&signalMask, SIGHUP);
+        sigprocmask(SIG_UNBLOCK, &signalMask, NULL);
 
         /* perhaps the right thing to to:
          create ptys .. and show console window when an application
@@ -1318,3 +1337,17 @@ void YApplication::setClipboardText(char *data, int len) {
         return ;
     fClip->setData(data, len);
 }
+
+bool YApplication::detectGNOME() {
+    Atom GNOME_NAME_SERVER(XInternAtom(display(), "GNOME_NAME_SERVER", true));
+
+    Atom r_type; int r_format;
+    unsigned long count, bytes_remain;
+    Window gnomeName;
+
+    return (GNOME_NAME_SERVER != None &&
+	    XGetWindowProperty(display(), desktop->handle(),
+			       GNOME_NAME_SERVER, 0, 1, false, XA_WINDOW,
+			       &r_type, &r_format, &count, &bytes_remain,
+			       (unsigned char **) &gnomeName) == Success);
+}
