@@ -4,6 +4,7 @@
 #include <signal.h>
 
 #include "ywindow.h"
+#include "ycursor.h"
 
 class YTimer;
 class YSocket;
@@ -20,6 +21,7 @@ public:
     
     //YRootWindow *root() { return fRoot; }
     Display *display() const { return fDisplay; }
+    bool detectGNOME();
 
     void saveEventTime(XEvent &xev);
     Time getEventTime() const { return lastEventTime; }
@@ -72,16 +74,24 @@ public:
 
     bool hasColormap();
 
+    static YCursor leftPointer;
+    static YCursor rightPointer;
+    static YCursor movePointer;
+
     unsigned int AltMask;
     unsigned int MetaMask;
     unsigned int NumLockMask;
     unsigned int ScrollLockMask;
     unsigned int SuperMask;
     unsigned int HyperMask;
+    unsigned int ModeSwitchMask;
 
     unsigned int KeyMask;
     unsigned int ButtonMask;
     unsigned int ButtonKeyMask;
+    
+    static char const * Name;
+
 private:
     Display *fDisplay;
     Time lastEventTime;
