@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997,1998 Marko Macek
+ * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
 
@@ -31,12 +31,9 @@ void YMenu::setActionListener(YActionListener *actionListener) {
 }
 
 void YMenu::finishPopup(YMenuItem *item, YAction *action, unsigned int modifiers) {
-    YActionListener *cmd = fActionListener;
-
     YPopupWindow::finishPopup();
 
-    if (item)
-        item->actionPerformed(cmd, action, modifiers);
+    if (item) item->actionPerformed(fActionListener, action, modifiers);
 }
 
 YTimer *YMenu::fMenuTimer = 0;
@@ -111,7 +108,7 @@ void YMenu::donePopup(YPopupWindow *popup) {
 
 bool YMenu::isCondCascade(int selItem) {
     if (selItem != -1 &&
-        item(selItem)->action() && item(selItem)->submenu())
+        item(selItem)->name() && item(selItem)->submenu())
     {
         return true;
     }
@@ -218,16 +215,19 @@ int YMenu::findActiveItem(int cur, int direction) {
     return c;
 }
 
+#ifdef DEBUG
 int YMenu::activateItem(int no, int byMouse, unsigned int modifiers) {
+#else
+int YMenu::activateItem(int /*no*/, int byMouse, unsigned int modifiers) {
+#endif
     PRECONDITION(selectedItem == no && selectedItem != -1);
     if (item(selectedItem)->isEnabled()) {
         if (item(selectedItem)->action() == 0 &&
             item(selectedItem)->submenu() != 0)
-        {
             focusItem(selectedItem, 1, byMouse);
-        } else if (item(selectedItem)->action()) {
-            finishPopup(item(selectedItem), item(selectedItem)->action(), modifiers);
-        }
+        else if (item(selectedItem)->action())
+            finishPopup(item(selectedItem),
+	    		item(selectedItem)->action(), modifiers);
     } else {
         //XBell(app->display(), 50);
         return -1;
@@ -294,8 +294,7 @@ bool YMenu::handleKey(const XKeyEvent &key) {
                 else if (k == XK_Return || k == XK_KP_Enter) {
                     if (selectedItem != -1 &&
                         (item(selectedItem)->action() != 0 ||
-                         item(selectedItem)->submenu() != 0))
-                    {
+                         item(selectedItem)->submenu() != 0)) {
                         activateItem(selectedItem, 0, key.state);
                         return true;
                     }
@@ -336,13 +335,11 @@ void YMenu::handleButton(const XButtonEvent &button) {
              item(selectedItem)->action() != 0)
             &&
             (item(selectedItem)->action() == 0 ||
-             !item(selectedItem)->submenu() || !nocascade) // ??? !!! ??? WTF
-           )
-        {
+             !item(selectedItem)->submenu() || !nocascade)) { // ??? !!! ??? WTF
             activatedX = button.x_root;
             activatedY = button.y_root;
             activateItem(selectedItem, 1, button.state);
-            return ;
+            return;
         }
         if (button.type == ButtonRelease &&
             (selectedItem == -1 || (item(selectedItem)->action() == 0 && item(selectedItem)->submenu() == 0)))
@@ -359,12 +356,10 @@ void YMenu::handleMotion(const XMotionEvent &motion) {
                          Button4Mask |
                          Button5Mask)) ? true : false;
 
-
     if (menuMouseTracking || isButton) {
         int selItem = findItem(motion.x_root - x(), motion.y_root - y());
-
         if (fMenuTimer && fMenuTimer->getTimerListener() == this) {
-            //printf("sel=%d timer=%d listener=%p =? this=%p\n", selItem, fTimerItem, fMenuTimer->getTimerListener(), this);
+            //msg("sel=%d timer=%d listener=%p =? this=%p", selItem, fTimerItem, fMenuTimer->getTimerListener(), this);
             if (selItem != fTimerItem || fTimerSlow) {
                 fTimerItem = -1;
                 if (fMenuTimer)
@@ -402,7 +397,6 @@ void YMenu::handleMotion(const XMotionEvent &motion) {
                     canFast = false;
             }
 
-
             if (canFast) {
                 YPopupWindow *p = fPopup;
 
@@ -464,7 +458,7 @@ void YMenu::handleMotion(const XMotionEvent &motion) {
     YPopupWindow::handleMotion(motion);
 }
 
-bool YMenu::handleTimer(YTimer *timer) {
+bool YMenu::handleTimer(YTimer */*timer*/) {
     activatedX = fTimerX;
     activatedY = fTimerY;
     focusItem(fTimerItem, fTimerSubmenu, 1);
@@ -519,6 +513,10 @@ YMenuItem * YMenu::addSeparator() {
     return add(new YMenuItem());
 }
 
+YMenuItem * YMenu::addLabel(const char *name) {
+    return add(new YMenuItem(name));
+}
+
 void YMenu::removeAll() {
     if (fPopup) {
         fPopup->popdown();
@@ -548,20 +546,31 @@ YMenuItem * YMenu::add(YMenuItem *item) {
     return item;
 }
 
-YMenuItem *YMenu::findAction(YAction *action) {
+YMenuItem *YMenu::findAction(const YAction *action) {
     for (int i = 0; i < itemCount(); i++)
         if (action == item(i)->action())
             return item(i);
     return 0;
 }
 
-YMenuItem *YMenu::findSubmenu(YMenu *sub) {
+YMenuItem *YMenu::findSubmenu(const YMenu *sub) {
     for (int i = 0; i < itemCount(); i++)
         if (sub == item(i)->submenu())
             return item(i);
     return 0;
 }
 
+YMenuItem *YMenu::findName(const char *name, const int first = 0) {
+    if (name != NULL)
+        for (int i = first; i < itemCount(); i++) {
+            const char *iname = item(i)->name();
+            if (iname && !strcmp(name, iname))
+                return item(i);
+        }
+
+    return 0;
+}
+
 void YMenu::enableCommand(YAction *action) {
     for (int i = 0; i < itemCount(); i++)
         if (action == 0 || action == item(i)->action())
@@ -580,7 +589,7 @@ int YMenu::getItemHeight(int itemNo, int &h, int &top, int &bottom, int &pad) {
     if (itemNo < 0 || itemNo > itemCount())
         return -1;
 
-    if (item(itemNo)->action() == 0 && item(itemNo)->submenu() == 0) {
+    if (item(itemNo)->name() == 0 && item(itemNo)->submenu() == 0) {
         top = 0;
         bottom = 0;
         pad = 1;
@@ -624,7 +633,7 @@ int YMenu::getItemHeight(int itemNo, int &h, int &top, int &bottom, int &pad) {
 void YMenu::getItemWidth(int i, int &iw, int &nw, int &pw) {
     iw = nw = pw = 0;
 
-    if (item(i)->action() == 0 && item(i)->submenu() == 0) {
+    if (item(i)->name() == 0 && item(i)->submenu() == 0) {
         iw = 0;
         nw = 0;
         pw = 0;
@@ -758,13 +767,14 @@ void YMenu::paintItems() {
         paintItem(g, i, l, t, r, (i == selectedItem || i == paintedItem) ? 1 : 0);
     paintedItem = selectedItem;
 }
+
 void YMenu::drawSeparator(Graphics &g, int x, int y, int w) {
     if (wmLook == lookMetal) {
         g.setColor(menuBg);
         g.drawLine(x, y + 0, w, y + 0);
         g.setColor(activeMenuItemBg);
         g.drawLine(x, y + 1, w, y + 1);;
-        g.setColor(YColor::white);
+        g.setColor(menuBg->brighter());
         g.drawLine(x, y + 2, w, y + 2);;
         g.drawLine(x, y, x, y + 2);
         g.setColor(menuBg);
@@ -788,17 +798,19 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
     const char *param = mitem->param();
 
     g.setColor(menuBg);
-    if (mitem->action() == 0 && mitem->submenu() == 0) {
+    if (mitem->name() == 0 && mitem->submenu() == 0) {
         if (paint)
             drawSeparator(g, 1, t, width() - 2);
         t += (wmLook == lookMetal) ? 3 : 4;
     } else {
         int eh, top, bottom, pad, ih;
+        bool active = (i == selectedItem && 
+	    (mitem->action () || mitem->submenu()));
 
         getItemHeight(i, eh, top, bottom, pad);
         ih = eh - top - bottom - pad - pad;
 
-        if (i == selectedItem)
+        if (active)
             g.setColor(activeMenuItemBg);
 
         if (paint) {
@@ -808,13 +820,13 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
                 g.fillRect(l, t, width() - r - l, eh);
 
             if (wmLook == lookMetal && i != selectedItem) {
-                g.setColor(YColor::white);
+                g.setColor(menuBg->brighter());
                 g.drawLine(1, t, 1, t + eh - 1);
                 g.setColor(menuBg);
             }
 
             if (wmLook != lookWin95 && wmLook != lookWarp4 &&
-                i == selectedItem)
+                active)
             {
                 bool raised = false;
 #ifdef OLDMOTIF
@@ -842,14 +854,14 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
             YColor *fg;
             if (!mitem->isEnabled())
                 fg = disabledMenuItemFg;
-            else if (i == selectedItem)
+            else if (active)
                 fg = activeMenuItemFg;
             else
                 fg = menuItemFg;
             g.setColor(fg);
             g.setFont(menuFont);
 
-            int delta = (i == selectedItem) ? 1 : 0;
+            int delta = (active) ? 1 : 0;
             if (wmLook == lookMotif || wmLook == lookGtk ||
                 wmLook == lookWarp4 || wmLook == lookWin95 ||
                 wmLook == lookMetal)
@@ -879,7 +891,7 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
 
             if (name) {
                 if (!mitem->isEnabled()) {
-                    g.setColor(YColor::white);
+                    g.setColor(menuBg->brighter());
                     g.drawChars(name, 0, strlen(name),
                                 1 + delta + namePos, 1 + baseLine);
 
@@ -900,7 +912,7 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
 
             if (param) {
                 if (!mitem->isEnabled()) {
-                    g.setColor(YColor::white);
+                    g.setColor(menuBg->brighter());
                     g.drawChars(param, 0, strlen(param),
                                 paramPos + delta + 1,
                                 baseLine + 1);
@@ -911,8 +923,8 @@ void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
                             baseLine);
             }
             else if (mitem->submenu() != 0) {
-                int active = ((mitem->action() == 0 && i == selectedItem) ||
-                              fPopup == mitem->submenu()) ? 1 : 0;
+//                int active = ((mitem->action() == 0 && i == selectedItem) ||
+//                              fPopup == mitem->submenu()) ? 1 : 0;
                 if (mitem->action()) {
                     g.setColor(menuBg);
                     if (0) {
@@ -962,7 +974,7 @@ void YMenu::paint(Graphics &g, int /*_x*/, int /*_y*/, unsigned int /*_width*/,
         g.drawLine(0, 0, 0, height() - 1);
         g.drawLine(width() - 1, 0, width() - 1, height() - 1);
         g.drawLine(0, height() - 1, width() - 1, height() - 1);
-        g.setColor(YColor::white);
+        g.setColor(menuBg->brighter());
         g.drawLine(1, 1, width() - 2, 1);
         g.setColor(menuBg);
         g.drawLine(1, height() - 2, width() - 2, height() - 2);
