@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1999 Marko Macek
+ * Copyright (C) 1999-2001 Marko Macek
  *
  * MessageBox
  */
@@ -17,6 +17,8 @@
 #include "wmframe.h"
 #include "sysdep.h"
 
+#include "intl.h"
+
 YMsgBox::YMsgBox(int buttons, YWindow *owner): YDialog(owner) {
     fListener = 0;
     fButtonOK = 0;
@@ -29,7 +31,7 @@ YMsgBox::YMsgBox(int buttons, YWindow *owner): YDialog(owner) {
     if (buttons & mbOK) {
         fButtonOK = new YActionButton(this);
         if (fButtonOK) {
-            fButtonOK->setText("OK");
+            fButtonOK->setText(_("OK"));
             fButtonOK->setActionListener(this);
             fButtonOK->show();
         }
@@ -37,7 +39,7 @@ YMsgBox::YMsgBox(int buttons, YWindow *owner): YDialog(owner) {
     if (buttons & mbCancel) {
         fButtonCancel = new YActionButton(this);
         if (fButtonCancel) {
-            fButtonCancel->setText("Cancel");
+            fButtonCancel->setText(_("Cancel"));
             fButtonCancel->setActionListener(this);
             fButtonCancel->show();
         }
@@ -84,11 +86,10 @@ void YMsgBox::getMaxButton(unsigned int &w, unsigned int &h) {
 }
 
 void YMsgBox::autoSize() {
-    int lw = fLabel ? fLabel->width() : 0;
-    int w = lw + 40, h;
+    unsigned lw = fLabel ? fLabel->width() : 0;
+    unsigned w = lw + 40, h;
 
-    if (w < 300)
-        w = 300;
+    w = clamp(w, 300U, desktop->width());
     
     h = 20;
     if (fLabel) {
@@ -135,7 +136,7 @@ void YMsgBox::setText(const char *text) {
     autoSize();
 }
 
-void YMsgBox::setPixmap(YPixmap *pixmap) {
+void YMsgBox::setPixmap(YPixmap */*pixmap*/) {
 }
 
 void YMsgBox::actionPerformed(YAction *action, unsigned int /*modifiers*/) {
@@ -152,7 +153,7 @@ void YMsgBox::handleClose() {
     fListener->handleMsgBox(this, 0);
 }
 
-void YMsgBox::handleFocus(const XFocusChangeEvent &focus) {
+void YMsgBox::handleFocus(const XFocusChangeEvent &/*focus*/) {
 }
 
 void YMsgBox::showFocused() {
@@ -162,8 +163,14 @@ void YMsgBox::showFocused() {
         getFrame()->setPosition(desktop->width() / 2 - getFrame()->width() / 2,
                                 desktop->height() / 2 - getFrame()->height() / 2);
         getFrame()->activate(true);
-        if (fButtonCancel) {
-            fButtonCancel->requestFocus();
+	
+	switch(msgBoxDefaultAction) {
+	    case 0:
+		if (fButtonCancel) fButtonCancel->requestFocus();
+		break;
+	    case 1:
+		if (fButtonOK) fButtonOK->requestFocus();
+		break;
         }
     }
 }
