@@ -38,16 +38,21 @@ struct XSelectionEvent;
 struct XSelectionClearEvent;
 struct XSelectionRequestEvent;
 #endif
+
 #ifndef __YIMP_XUTIL__
 #ifdef SHAPE
-struct XShapeEvent;
 struct XTextProperty;
 #endif
 #endif
 
+#ifdef SHAPE
+struct XShapeEvent;
+#endif
+
 class YColor {
 public:
     YColor(unsigned short red, unsigned short green, unsigned short blue);
+    YColor(unsigned long pixel);
     YColor(const char *clr);
 
     void alloc();
@@ -87,6 +92,10 @@ private:
     int fontAscent, fontDescent;
 
     YFont(const char *name);
+#ifdef I18N
+    void GetFontNameElement(const char *, char *, int, int);
+    XFontSet CreateFontSetWithGuess(Display *, const char *, char ***, int *, char **);
+#endif
 
     void alloc();
 
@@ -99,6 +108,9 @@ public:
     YPixmap(const char *fileName, int w, int h);
     YPixmap(int w, int h, bool mask = false);
     YPixmap(Pixmap pixmap, Pixmap mask, int w, int h);
+#ifdef CONFIG_IMLIB
+    YPixmap(Pixmap pixmap, Pixmap mask, int w, int h, int hScaled, int hScaled);
+#endif
     ~YPixmap();
 
     Pixmap pixmap() const { return fPixmap; }
