@@ -2,6 +2,8 @@
 #define __YWINDOW_H
 
 #include "ypaint.h"
+#include "ycursor.h"
+
 class YPopupWindow;
 class YToolTip;
 class YTimer;
@@ -25,6 +27,9 @@ public:
     void reparent(YWindow *parent, int x, int y);
 
     void setWindowFocus();
+    
+    void setTitle(char const * title);
+    void setClassHint(char const * rName, char const * rClass);
 
     void setGeometry(int x, int y, unsigned int width, unsigned int height);
     void setSize(unsigned int width, unsigned int height);
@@ -74,8 +79,8 @@ public:
     virtual bool handleAutoScroll(const XMotionEvent &mouse);
     void beginAutoScroll(bool autoScroll, const XMotionEvent *motion);
 
-    void setPointer(Cursor pointer);
-    void setGrabPointer(Cursor pointer);
+    void setPointer(const YCursor& pointer);
+    void setGrabPointer(const YCursor& pointer);
     void grabKeyM(int key, unsigned int modifiers);
     void grabKey(int key, unsigned int modifiers);
     void grabVKey(int key, unsigned int vmodifiers);
@@ -193,7 +198,7 @@ private:
     unsigned long fStyle;
     int fX, fY;
     unsigned int fWidth, fHeight;
-    Cursor fPointer;
+    YCursor fPointer;
     int unmapCount;
     Graphics *fGraphics;
     long fEventMask;
